=== modified file 'README.txt'
--- README.txt	2008-03-03 23:10:35 +0000
+++ README.txt	2008-06-16 09:48:20 +0000
@@ -52,23 +52,12 @@
 locking to prevent corruption.
 
 
-CACHES
-------
-
-To speed up operations with large branches, loggerhead can be configured
-to create three caches per branch when it first launches:
-
-    - a revision data cache
-    - a cache of files changed in a revision
-    - a text searching cache
-
-You can put the cache folder anywhere.
-
-The revision data cache and text searching cache will be automatically
-filled once loggerhead starts up.  The caches for a branch with 15,000
-revisions take about four minutes each on a fairly fast machine, but
-mileage may vary.  Once they are built, they update every six hours or
-so but usually finish quickly (or instantly) after the initial creation.
+FILES CHANGED CACHE
+-------------------
+
+To speed up the display of the changelog view for large trees,
+loggerhead can be configured to cache the files changes between
+revisions.  Set the 'cachepath' value in the config file.
 
 
 SUPPORT

=== modified file 'loggerhead.conf.example'
--- loggerhead.conf.example	2008-03-06 07:30:37 +0000
+++ loggerhead.conf.example	2008-06-16 09:48:20 +0000
@@ -27,8 +27,8 @@
     # for the top browsing page, you can include a description, in raw HTML.
     #description = 'Bazaar is a <i>revision control system</i>.'
 
-    # should we cache revisions and build a text index?  (HIGHLY recommended)
-    # this will be a folder containing a few files.  if the folder doesn't
+    # should we cache files changed? (recommended for large trees)
+    # this will be a folder containing one file.  if the folder doesn't
     # exist, it'll be created.
     #
     # all branches in this project will share the same cache, which is usually

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2006-12-11 06:44:19 +0000
+++ loggerhead/__init__.py	2008-06-10 22:44:49 +0000
@@ -0,0 +1,2 @@
+# Register the 'zpt' template handler:
+import turbosimpletal

=== modified file 'loggerhead/branchview.py'
--- loggerhead/branchview.py	2008-02-27 03:36:09 +0000
+++ loggerhead/branchview.py	2008-06-16 09:26:05 +0000
@@ -29,9 +29,8 @@
 from cherrypy import HTTPRedirect
 
 from loggerhead import util
-from loggerhead.changecache import ChangeCache, FileChangeCache
+from loggerhead.changecache import FileChangeCache
 from loggerhead.history import History
-from loggerhead.textindex import TextIndex
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.revision_ui import RevisionUI
@@ -45,19 +44,20 @@
 
 
 class BranchView (object):
-    def __init__(self, group_name, name, subfolder, absfolder, config, project_config):
+    def __init__(self, group_name, name, subfolder, absfolder, config,
+                 project_config, root_config):
         self._group_name = group_name
         self._name = name
         self._folder = subfolder
         self._absfolder = absfolder
         self._config = config
         self._project_config = project_config
+        self._root_config = root_config
         self.log = logging.getLogger('loggerhead.%s' % (name,))
 
         # branch history
         self._history_lock = threading.RLock()
         self._history = None
-        self._closed = False
 
         self.changes = ChangeLogUI(self)
         self.revision = RevisionUI(self)
@@ -70,17 +70,6 @@
         # force history object to be loaded:
         self.get_history()
 
-        turbogears.startup.call_on_shutdown.append(self.close)
-
-    @with_history_lock
-    def close(self):
-        # it's important that we cleanly detach the history, so the cache
-        # files can be closed correctly and hopefully remain uncorrupted.
-        # this should also stop any ongoing indexing.
-        self._history.detach()
-        self._history = None
-        self._closed = True
-
     config = property(lambda self: self._config)
 
     name = property(lambda self: self._name)
@@ -127,6 +116,12 @@
     def index(self):
         raise HTTPRedirect(self.url('/changes'))
 
+    def get_config_item(self, item, default=None):
+        for conf in self._config, self._project_config, self._root_config:
+            if item in conf:
+                return conf[item]
+        return default
+
     @with_history_lock
     def get_history(self):
         """
@@ -135,12 +130,8 @@
         calls.  but if the bazaar branch on-disk has been updated since this
         History was created, a new object will be created and returned.
         """
-        if self._closed:
-            return None
         if (self._history is None) or self._history.out_of_date():
             self.log.debug('Reload branch history...')
-            if self._history is not None:
-                self._history.detach()
             _history = self._history = History.from_folder(
                 self._absfolder, self._name)
             cache_path = self._config.get('cachepath', None)
@@ -148,16 +139,9 @@
                 # try the project config
                 cache_path = self._project_config.get('cachepath', None)
             if cache_path is not None:
-                _history.use_cache(ChangeCache(_history, cache_path))
                 _history.use_file_cache(FileChangeCache(_history, cache_path))
-                _history.use_search_index(TextIndex(_history, cache_path))
         return self._history
 
-    def check_rebuild(self):
-        h = self.get_history()
-        if h is not None:
-            h.check_rebuild()
-
     def url(self, elements, **kw):
         "build an url relative to this branch"
         if not isinstance(elements, list):

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2008-02-27 03:36:09 +0000
+++ loggerhead/changecache.py	2008-06-16 09:07:39 +0000
@@ -50,8 +50,6 @@
                 ).replace('?', _param_marker)
 _insert_stmt = ("insert into revisiondata (revid, data) "
                 "values (?, ?)").replace('?', _param_marker)
-_update_stmt = ("update revisiondata set data = ? where revid = ?"
-                ).replace('?', _param_marker)
 
 
 
@@ -85,11 +83,6 @@
             self.cursor.execute(_insert_stmt, (r, self._serialize(d)))
         if commit:
             self.connection.commit()
-    def update(self, revid_obj_pairs, commit=True):
-        for  (r, d) in revid_obj_pairs:
-            self.cursor.execute(_update_stmt, (self._serialize(d), r))
-        if commit:
-            self.connection.commit()
     def count(self):
         self.cursor.execute(
             "select count(*) from revisiondata")
@@ -99,124 +92,6 @@
             self.connection.commit()
         self.connection.close()
 
-class ChangeCache (object):
-
-    def __init__(self, history, cache_path):
-        self.history = history
-        self.log = history.log
-
-        if not os.path.exists(cache_path):
-            os.mkdir(cache_path)
-
-        self._changes_filename = os.path.join(cache_path, 'changes.sql')
-
-        # use a lockfile since the cache folder could be shared across different processes.
-        self._lock = LockFile(os.path.join(cache_path, 'lock'))
-        self._closed = False
-
-##         # this is fluff; don't slow down startup time with it.
-##         # but it is racy in tests :(
-##         def log_sizes():
-##             self.log.info('Using change cache %s; %d entries.' % (cache_path, self.size()))
-##         threading.Thread(target=log_sizes).start()
-
-    def _cache(self):
-        return FakeShelf(self._changes_filename)
-
-    @with_lock
-    def close(self):
-        self.log.debug('Closing cache file.')
-        self._closed = True
-
-    @with_lock
-    def closed(self):
-        return self._closed
-
-    @with_lock
-    def flush(self):
-        pass
-
-    @with_lock
-    def get_changes(self, revid_list):
-        """
-        get a list of changes by their revision_ids.  any changes missing
-        from the cache are fetched by calling L{History.get_change_uncached}
-        and inserted into the cache before returning.
-        """
-        out = []
-        missing_revids = []
-        missing_revid_indices = []
-        cache = self._cache()
-        for revid in revid_list:
-            entry = cache.get(revid)
-            if entry is not None:
-                out.append(entry)
-            else:
-                missing_revids.append(revid)
-                missing_revid_indices.append(len(out))
-                out.append(None)
-        if missing_revids:
-            missing_entries = self.history.get_changes_uncached(missing_revids)
-            missing_entry_dict = {}
-            for entry in missing_entries:
-                missing_entry_dict[entry.revid] = entry
-            revid_entry_pairs = []
-            for i, revid in zip(missing_revid_indices, missing_revids):
-                out[i] = entry = missing_entry_dict.get(revid)
-                if entry is not None:
-                    revid_entry_pairs.append((revid, entry))
-            cache.add(revid_entry_pairs)
-        return filter(None, out)
-
-    @with_lock
-    def full(self):
-        cache = self._cache()
-        last_revid = util.to_utf8(self.history.last_revid)
-        revision_history = self.history.get_revision_history()
-        return (cache.count() >= len(revision_history)
-                and cache.get(last_revid) is not None)
-
-    @with_lock
-    def size(self):
-        return self._cache().count()
-
-    def check_rebuild(self, max_time=3600):
-        """
-        check if we need to fill in any missing pieces of the cache.  pull in
-        any missing changes, but don't work any longer than C{max_time}
-        seconds.
-        """
-        if self.closed() or self.full():
-            return
-
-        self.log.info('Building revision cache...')
-        start_time = time.time()
-        last_update = time.time()
-        count = 0
-
-        work = list(self.history.get_revision_history())
-        jump = 100
-        for i in xrange(0, len(work), jump):
-            r = work[i:i + jump]
-            # must call into history so we grab the branch lock (otherwise, lock inversion)
-            self.history.get_changes(r)
-            if self.closed():
-                self.flush()
-                return
-            count += jump
-            now = time.time()
-            if now - start_time > max_time:
-                self.log.info('Cache rebuilding will pause for now.')
-                self.flush()
-                return
-            if now - last_update > 60:
-                self.log.info('Revision cache rebuilding continues: %d/%d' % (min(count, len(work)), len(work)))
-                last_update = time.time()
-                self.flush()
-            # give someone else a chance at the lock
-            time.sleep(1)
-        self.log.info('Revision cache rebuild completed.')
-        self.flush()
 
 class FileChangeCache(object):
     def __init__(self, history, cache_path):

=== modified file 'loggerhead/config/app.cfg'
--- loggerhead/config/app.cfg	2006-12-11 06:44:19 +0000
+++ loggerhead/config/app.cfg	2008-06-10 04:46:49 +0000
@@ -8,17 +8,6 @@
 
 # VIEW
 
-# which view (template engine) to use if one is not specified in the
-# template name
-# tg.defaultview = "kid"
-
-# The following kid settings determine the settings used by the kid serializer.
-
-# One of (html|xml|json)
-# kid.outputformat="html"
-
-# kid.encoding="utf-8"
-
 # The sitetemplate is used for overall styling of a site that
 # includes multiple TurboGears applications
 # tg.sitetemplate="<packagename.templates.templatename>"

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2008-02-27 03:35:24 +0000
+++ loggerhead/controllers/__init__.py	2008-06-16 09:28:18 +0000
@@ -28,6 +28,7 @@
 from loggerhead import util
 from loggerhead.branchview import BranchView
 from loggerhead.history import is_branch
+from loggerhead.templatefunctions import templatefunctions
 
 log = logging.getLogger("loggerhead.controllers")
 
@@ -40,12 +41,13 @@
 
 
 class Project (object):
-    def __init__(self, name, config):
+    def __init__(self, name, config, root_config):
         self.name = name
         self.friendly_name = config.get('name', name)
         self.description = config.get('description', '')
         self.long_description = config.get('long_description', '')
         self._config = config
+        self._root_config = root_config
         
         self._views = []
         for view_name in config.sections:
@@ -83,7 +85,9 @@
         
     def _add_view(self, view_name, view_config, folder):
         c_view_name = cherrypy_friendly(view_name)
-        view = BranchView(self.name, c_view_name, view_name, folder, view_config, self._config)
+        view = BranchView(
+            self.name, c_view_name, view_name, folder, view_config,
+            self._config, self._root_config)
         self._views.append(view)
         setattr(self, c_view_name, view)
         
@@ -96,29 +100,19 @@
         self._config = config
         for project_name in self._config.sections:
             c_project_name = cherrypy_friendly(project_name)
-            project = Project(c_project_name, self._config[project_name])
+            project = Project(
+                c_project_name, self._config[project_name], self._config)
             self._projects.append(project)
             setattr(self, c_project_name, project)
         
-    @turbogears.expose(template='loggerhead.templates.browse')
+    @turbogears.expose(template='zpt:loggerhead.templates.browse')
     def index(self):
         for p in self._projects:
             p._recheck_auto_folders()
-        return {
+        vals = {
             'projects': self._projects,
             'util': util,
-            'title': self._config.get('title', ''),
+            'title': self._config.get('title', None),
         }
-
-    def _check_rebuild(self):
-        for p in self._projects:
-            for v in p.views:
-                v.check_rebuild()
-
-
-
-# for use in profiling the very-slow get_change() method:
-#h = Root.bazaar.bzr_dev.get_history()
-#w = list(h.get_revision_history())
-#h._get_changes_profiled(w[:100])
-
+        vals.update(templatefunctions)
+        return vals

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-02-27 03:35:24 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-06-16 08:49:28 +0000
@@ -26,6 +26,7 @@
 from cherrypy import HTTPError
 
 from loggerhead import util
+from loggerhead.templatefunctions import templatefunctions
 
 
 log = logging.getLogger("loggerhead.controllers")
@@ -45,7 +46,7 @@
         self.log = branch.log
 
     @util.strip_whitespace
-    @turbogears.expose(html='loggerhead.templates.annotate')
+    @turbogears.expose(html='zpt:loggerhead.templates.annotate')
     def default(self, *args, **kw):
         z = time.time()
         h = self._branch.get_history()
@@ -89,8 +90,9 @@
                 'navigation': navigation,
                 'change': h.get_changes([ revid ])[0],
                 'contents': list(h.annotate_file(file_id, revid)),
+                'url': self._branch.context_url,
             }
-            h.flush_cache()
+            vals.update(templatefunctions)
             self.log.info('/annotate: %r secs' % (time.time() - z,))
             return vals
         finally:

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2008-02-27 03:35:24 +0000
+++ loggerhead/controllers/atom_ui.py	2008-06-16 08:49:28 +0000
@@ -20,6 +20,7 @@
 import turbogears
 
 from loggerhead import util
+from loggerhead.templatefunctions import templatefunctions
 
 
 class AtomUI (object):
@@ -29,8 +30,9 @@
         self._branch = branch
         self.log = branch.log
 
-    @turbogears.expose(template='loggerhead.templates.atom', format="xml", content_type="application/atom+xml")
-    def default(self, *args):
+    @turbogears.expose(template='zpt:loggerhead.templates.atom',
+                       format="xml", content_type="application/atom+xml")
+    def default(self, *args, **kwargs):
         h = self._branch.get_history()
 
         h._branch.lock_read()
@@ -47,7 +49,7 @@
                 'history': h,
                 'updated': entries[0].date.isoformat() + 'Z',
             }
-            h.flush_cache()
+            vals.update(templatefunctions)
             return vals
         finally:
             h._branch.unlock()

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-03-10 04:02:42 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-06-16 08:49:28 +0000
@@ -23,6 +23,7 @@
 from cherrypy import InternalError
 
 from loggerhead import util
+from loggerhead.templatefunctions import templatefunctions
 
 
 class ChangeLogUI (object):
@@ -33,7 +34,7 @@
         self.log = branch.log
         
     @util.strip_whitespace
-    @turbogears.expose(html='loggerhead.templates.changelog')
+    @turbogears.expose(html='zpt:loggerhead.templates.changelog')
     def default(self, *args, **kw):
         z = time.time()
         h = self._branch.get_history()
@@ -100,6 +101,9 @@
                         all_same_author = False
                         break
 
+            def url(pathargs, **kw):
+                return self._branch.url(pathargs, **util.get_context(**kw))
+
             vals = {
                 'branch': self._branch,
                 'changes': changes,
@@ -113,8 +117,9 @@
                 'query': query,
                 'search_failed': search_failed,
                 'all_same_author': all_same_author,
+                'url': self._branch.context_url,
             }
-            h.flush_cache()
+            vals.update(templatefunctions)
             self.log.info('/changes %r: %r secs' % (revid, time.time() - z))
             return vals
         finally:

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2008-03-10 04:19:01 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-06-16 08:49:28 +0000
@@ -25,6 +25,7 @@
 from cherrypy import InternalError
 
 from loggerhead import util
+from loggerhead.templatefunctions import templatefunctions
 
 
 log = logging.getLogger("loggerhead.controllers")
@@ -44,7 +45,7 @@
         self.log = branch.log
 
     @util.strip_whitespace
-    @turbogears.expose(html='loggerhead.templates.inventory')
+    @turbogears.expose(html='zpt:loggerhead.templates.inventory')
     def default(self, *args, **kw):
         z = time.time()
         h = self._branch.get_history()
@@ -100,9 +101,10 @@
                 'history': h,
                 'posixpath': posixpath,
                 'navigation': navigation,
+                'url': self._branch.context_url,
                 'start_revid': start_revid,
             }
-            h.flush_cache()
+            vals.update(templatefunctions)
             self.log.info('/inventory %r: %r secs' % (revid, time.time() - z))
             return vals
         finally:

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2008-03-10 04:02:42 +0000
+++ loggerhead/controllers/revision_ui.py	2008-06-16 08:49:28 +0000
@@ -23,6 +23,10 @@
 from cherrypy import InternalError
 
 from loggerhead import util
+from loggerhead.templatefunctions import templatefunctions
+
+
+DEFAULT_LINE_COUNT_LIMIT = 3000
 
 
 class RevisionUI (object):
@@ -34,7 +38,7 @@
     
 #    @util.lsprof
     @util.strip_whitespace
-    @turbogears.expose(html='loggerhead.templates.revision')
+    @turbogears.expose(html='zpt:loggerhead.templates.revision')
     def default(self, *args, **kw):
         z = time.time()
         h = self._branch.get_history()
@@ -71,6 +75,13 @@
             # add parent & merge-point branch-nick info, in case it's useful
             h.get_branch_nicks([ change ])
 
+            line_count_limit = int(self._branch.get_config_item(
+                'line_count_limit', DEFAULT_LINE_COUNT_LIMIT))
+            line_count = 0
+            for file in change.changes.modified:
+                for chunk in file.chunks:
+                    line_count += len(chunk.diff)
+
             # let's make side-by-side diff be the default
             side_by_side = not kw.get('unified', False)
             if side_by_side:
@@ -89,8 +100,12 @@
                 'remember': remember,
                 'compare_revid': compare_revid,
                 'side_by_side': side_by_side,
+                'url': self._branch.context_url,
+                'line_count': line_count,
+                'line_count_limit': line_count_limit,
+                'show_plain_diffs': line_count > line_count_limit,
             }
-            h.flush_cache()
+            vals.update(templatefunctions)
             self.log.info('/revision: %r seconds' % (time.time() - z,))
             return vals
         finally:

=== modified file 'loggerhead/daemon.py'
--- loggerhead/daemon.py	2008-02-27 03:36:09 +0000
+++ loggerhead/daemon.py	2008-04-15 03:10:32 +0000
@@ -9,7 +9,6 @@
     background as a daemon.
     """
 
-    UMASK = 0
     WORKDIR = "/"
     REDIRECT_TO = getattr(os, 'devnull', '/dev/null')
     MAXFD = 1024
@@ -19,7 +18,7 @@
     except OSError, e:
         raise Exception("%s [%d]" % (e.strerror, e.errno))
 
-    if (pid == 0):      # The first child.
+    if pid == 0:      # The first child.
         os.setsid()
 
         try:
@@ -27,33 +26,30 @@
         except OSError, e:
             raise Exception, "%s [%d]" % (e.strerror, e.errno)
 
-        if (pid == 0):  # The second child.
+        if pid == 0:  # The second child.
             os.chdir(WORKDIR)
-            os.umask(UMASK)
         else:
             os._exit(0) # Exit parent (the first child) of the second child.
     else:
         os._exit(0)     # Exit parent of the first child.
 
-    #import resource            # Resource usage information.
-    #maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
-    #if (maxfd == resource.RLIM_INFINITY):
-    #    maxfd = MAXFD
-
-    # Iterate through and close all file descriptors.
-    #for fd in range(3, maxfd):
-    #    try:
-    #        os.close(fd)
-    #    except OSError:        # ERROR, fd wasn't open to begin with (ignored)
-    #        pass
-
-    # This call to open is guaranteed to return the lowest file descriptor,
-    # which will be 0 (stdin), since it was closed above.
-    #os.open(REDIRECT_TO, os.O_RDWR)    # standard input (0)
-
-    # Duplicate standard input to standard output and standard error.
-    #os.dup2(0, 1)                      # standard output (1)
-    #os.dup2(0, 2)                      # standard error (2)
+    import resource            # Resource usage information.
+    maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
+    if (maxfd == resource.RLIM_INFINITY):
+        maxfd = MAXFD
+
+    fd = os.open(REDIRECT_TO, os.O_RDONLY)
+    os.dup2(fd, 0)
+    fd = os.open(REDIRECT_TO, os.O_WRONLY)
+    os.dup2(fd, 1)
+    os.dup2(fd, 2)
+
+    # Iterate through and close all other file descriptors.
+    for fd in range(3, maxfd):
+        try:
+            os.close(fd)
+        except OSError:        # ERROR, fd wasn't open to begin with (ignored)
+            pass
 
     f = open(pidfile, 'w')
     f.write('%d\n' % (os.getpid(),))

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-03-10 04:17:25 +0000
+++ loggerhead/history.py	2008-06-16 09:48:20 +0000
@@ -155,8 +155,8 @@
 
     # Make short form of commit message.
     short_message = message[0]
-    if len(short_message) > 80:
-        short_message = short_message[:80] + '...'
+    if len(short_message) > 60:
+        short_message = short_message[:60] + '...'
 
     return message, short_message
 
@@ -191,9 +191,7 @@
 class History (object):
 
     def __init__(self):
-        self._change_cache = None
         self._file_change_cache = None
-        self._index = None
         self._lock = threading.RLock()
 
     @classmethod
@@ -202,13 +200,17 @@
         self = cls()
         self._branch = branch
         self._last_revid = self._branch.last_revision()
-        self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
 
         if name is None:
             name = self._branch.nick
         self._name = name
         self.log = logging.getLogger('loggerhead.%s' % (name,))
 
+        graph = branch.repository.get_graph()
+        parent_map = dict(((key, value) for key, value in
+             graph.iter_ancestry([self._last_revid]) if value is not None))
+
+        self._revision_graph = self._strip_NULL_ghosts(parent_map)
         self._full_history = []
         self._revision_info = {}
         self._revno_revid = {}
@@ -217,13 +219,17 @@
         else:
             self._merge_sort = bzrlib.tsort.merge_sort(
                 self._revision_graph, self._last_revid, generate_revno=True)
+
         for (seq, revid, merge_depth, revno, end_of_merge) in self._merge_sort:
             self._full_history.append(revid)
             revno_str = '.'.join(str(n) for n in revno)
             self._revno_revid[revno_str] = revid
-            self._revision_info[revid] = (seq, revid, merge_depth, revno_str, end_of_merge)
+            self._revision_info[revid] = (
+                seq, revid, merge_depth, revno_str, end_of_merge)
+
         # cache merge info
         self._where_merged = {}
+
         for revid in self._revision_graph.keys():
             if self._revision_info[revid][2] == 0:
                 continue
@@ -233,6 +239,21 @@
         self.log.info('built revision graph cache: %r secs' % (time.time() - z,))
         return self
 
+    @staticmethod
+    def _strip_NULL_ghosts(revision_graph):
+        """
+        Copied over from bzrlib meant as a temporary workaround deprecated 
+        methods.
+        """
+
+        # Filter ghosts, and null:
+        if bzrlib.revision.NULL_REVISION in revision_graph:
+            del revision_graph[bzrlib.revision.NULL_REVISION]
+        for key, parents in revision_graph.items():
+            revision_graph[key] = tuple(parent for parent in parents if parent
+                in revision_graph)
+        return revision_graph
+
     @classmethod
     def from_folder(cls, path, name=None):
         b = bzrlib.branch.Branch.open(path)
@@ -254,50 +275,20 @@
             return True
         return self._branch.last_revision() != self._last_revid
 
-    def use_cache(self, cache):
-        self._change_cache = cache
-
     def use_file_cache(self, cache):
         self._file_change_cache = cache
 
-    def use_search_index(self, index):
-        self._index = index
-
     @property
     def has_revisions(self):
         return not bzrlib.revision.is_null(self.last_revid)
 
-    @with_branch_lock
-    def detach(self):
-        # called when a new history object needs to be created, because the
-        # branch history has changed.  we need to immediately close and stop
-        # using our caches, because a new history object will be created to
-        # replace us, using the same cache files.
-        # (may also be called during server shutdown.)
-        if self._change_cache is not None:
-            self._change_cache.close()
-            self._change_cache = None
-        if self._index is not None:
-            self._index.close()
-            self._index = None
-
-    def flush_cache(self):
-        if self._change_cache is None:
-            return
-        self._change_cache.flush()
-
-    def check_rebuild(self):
-        if self._change_cache is not None:
-            self._change_cache.check_rebuild()
-        if self._index is not None:
-            self._index.check_rebuild()
-
     last_revid = property(lambda self: self._last_revid, None, None)
 
     @with_branch_lock
     def get_config(self):
         return self._branch.get_config()
 
+
     def get_revno(self, revid):
         if revid not in self._revision_info:
             # ghost parent?
@@ -326,6 +317,8 @@
                 i += 1
             return r
         while 1:
+            if bzrlib.revision.is_null(revid):
+                return
             if introduced_revisions(revid) & revid_set:
                 yield revid
             parents = self._revision_graph[revid]
@@ -359,31 +352,6 @@
         return revid_list[index:]
 
     @with_branch_lock
-    def get_revision_history_matching(self, revid_list, text):
-        self.log.debug('searching %d revisions for %r', len(revid_list), text)
-        z = time.time()
-        # this is going to be painfully slow. :(
-        out = []
-        text = text.lower()
-        for revid in revid_list:
-            change = self.get_changes([ revid ])[0]
-            if text in change.comment.lower():
-                out.append(revid)
-        self.log.debug('searched %d revisions for %r in %r secs', len(revid_list), text, time.time() - z)
-        return out
-
-    def get_revision_history_matching_indexed(self, revid_list, text):
-        self.log.debug('searching %d revisions for %r', len(revid_list), text)
-        z = time.time()
-        if self._index is None:
-            return self.get_revision_history_matching(revid_list, text)
-        out = self._index.find(text, revid_list)
-        self.log.debug('searched %d revisions for %r in %r secs: %d results', len(revid_list), text, time.time() - z, len(out))
-        # put them in some coherent order :)
-        out = [r for r in self._full_history if r in out]
-        return out
-
-    @with_branch_lock
     def get_search_revid_list(self, query, revid_list):
         """
         given a "quick-search" query, try a few obvious possible meanings:
@@ -425,11 +393,6 @@
                 revid_list = list(self.get_revids_from(None, self._last_revid))
             return self.get_revision_history_since(revid_list, date)
 
-        # check comment fields.
-        if revid_list is None:
-            revid_list = self._full_history
-        return self.get_revision_history_matching_indexed(revid_list, query)
-
     revno_re = re.compile(r'^[\d\.]+$')
     # the date regex are without a final '$' so that queries like
     # "2006-11-30 12:15" still mostly work.  (i think it's better to give
@@ -503,7 +466,7 @@
             if revid not in revid_list:
                 # if the given revid is not in the revlist, use a revlist that
                 # starts at the given revid.
-                revid_list= self.get_file_view(revid, file_id)
+                revid_list = self.get_file_view(revid, file_id)
                 start_revid = revid
             return revid, start_revid, revid_list
 
@@ -514,12 +477,11 @@
             revid_list = None
 
         revid_list = self.get_search_revid_list(query, revid_list)
-        if len(revid_list) > 0:
+        if revid_list and len(revid_list) > 0:
             if revid not in revid_list:
                 revid = revid_list[0]
             return revid, start_revid, revid_list
         else:
-            # no results
             return None, None, []
 
     @with_branch_lock
@@ -541,7 +503,6 @@
             path = '/' + path
         return self._branch.repository.get_revision_inventory(revid).path2id(path)
 
-
     def get_merge_point_list(self, revid):
         """
         Return the list of revids that have merged this node.
@@ -621,10 +582,7 @@
 
         Revisions not present and NULL_REVISION will be ignored.
         """
-        if self._change_cache is None:
-            changes = self.get_changes_uncached(revid_list)
-        else:
-            changes = self._change_cache.get_changes(revid_list)
+        changes = self.get_changes_uncached(revid_list)
         if len(changes) == 0:
             return changes
 
@@ -634,11 +592,8 @@
             merge_revids = self.simplify_merge_point_list(self.get_merge_point_list(change.revid))
             change.merge_points = [util.Container(revid=r, revno=self.get_revno(r)) for r in merge_revids]
             if len(change.parents) > 0:
-                if isinstance(change.parents[0], util.Container):
-                    # old cache stored a potentially-bogus revno
-                    change.parents = [util.Container(revid=p.revid, revno=self.get_revno(p.revid)) for p in change.parents]
-                else:
-                    change.parents = [util.Container(revid=r, revno=self.get_revno(r)) for r in change.parents]
+                change.parents = [util.Container(revid=r, 
+                    revno=self.get_revno(r)) for r in change.parents]
             change.revno = self.get_revno(change.revid)
 
         parity = 0
@@ -648,29 +603,17 @@
 
         return changes
 
-    # alright, let's profile this sucka. (FIXME remove this eventually...)
-    def _get_changes_profiled(self, revid_list):
-        from loggerhead.lsprof import profile
-        import cPickle
-        ret, stats = profile(self.get_changes_uncached, revid_list)
-        stats.sort()
-        stats.freeze()
-        cPickle.dump(stats, open('lsprof.stats', 'w'), 2)
-        self.log.info('lsprof complete!')
-        return ret
-
     @with_branch_lock
     @with_bzrlib_read_lock
     def get_changes_uncached(self, revid_list):
         revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
                             revid_list)
-        repo = self._branch.repository
-        parent_map = repo.get_graph().get_parent_map(revid_list)
+        parent_map = self._branch.repository.get_graph().get_parent_map(revid_list)
         # We need to return the answer in the same order as the input,
         # less any ghosts.
         present_revids = [revid for revid in revid_list
                           if revid in parent_map]
-        rev_list = repo.get_revisions(present_revids)
+        rev_list = self._branch.repository.get_revisions(present_revids)
 
         return [self._change_from_revision(rev) for rev in rev_list]
 
@@ -814,7 +757,7 @@
                     diff = buffer.getvalue()
             else:
                 diff = ''
-            out.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid, chunks=self._process_diff(diff)))
+            out.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid, chunks=self._process_diff(diff), raw_diff=diff))
 
         return out
 
@@ -954,14 +897,10 @@
 
         file_revid = self.get_inventory(revid)[file_id].revision
         oldvalues = None
-
-        # because we cache revision metadata ourselves, it's actually much
-        # faster to call 'annotate_iter' on the weave directly than it is to
-        # ask bzrlib to annotate for us.
-        w = self._branch.repository.weave_store.get_weave(file_id, self._branch.repository.get_transaction())
-
+        tree = self._branch.repository.revision_tree(file_revid)
         revid_set = set()
-        for line_revid, text in w.annotate_iter(file_revid):
+
+        for line_revid, text in tree.annotate_iter(file_id):
             revid_set.add(line_revid)
             if self._BADCHARS_RE.match(text):
                 # bail out; this isn't displayable text
@@ -969,10 +908,11 @@
                                      text='(This is a binary file.)',
                                      change=util.Container())
                 return
-        change_cache = dict([(c.revid, c) for c in self.get_changes(list(revid_set))])
+        change_cache = dict([(c.revid, c) \
+                for c in self.get_changes(list(revid_set))])
 
         last_line_revid = None
-        for line_revid, text in w.annotate_iter(file_revid):
+        for line_revid, text in tree.annotate_iter(file_id):
             if line_revid == last_line_revid:
                 # remember which lines have a new revno and which don't
                 status = 'same'

=== removed file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2007-09-04 01:47:51 +0000
+++ loggerhead/static/css/style.css	1970-01-01 00:00:00 +0000
@@ -1,653 +0,0 @@
-
-/* from launchpad */
-body {
-    background-color: #fcfcfc;
-    color: black;
-    font-family: "Lucida Grande", "Trebuchet MS", sans-serif;
-    font-size: 12pt;
-    padding: 0px;
-    text-align: left;
-}
-
-a {
-    text-decoration: none;
-    color: #000088;
-}
-a:hover {
-    color: black;
-}
-
-img {
-    border: none;
-}
-
-h1 {
-    color: #708a96;
-    font-weight: normal;
-    font-size: 1.6em;
-    margin-bottom: 1em;
-}
-h1 .filename {
-    font-family: monospace;
-}
-
-.branch-name {
-    color: #47626e;
-}
-
-.footer {
-    margin-top: 1em;
-    margin-bottom: 1em;
-}
-.footer .banner {
-    margin-right: 5px;
-}
-.footer .banner img {
-    border: 1px solid #8cacbb;
-}
-
-.collapse-triangle {
-    padding-right: 0.5em;
-}
-
-.hide-all {
-    font-size: smaller;
-    padding: 0.25em 0.75em 0.25em 0.75em;
-}
-
-
-
-/* ----- navigation bar at the top ----- */
-
-.navbar {
-}
-
-.bar table {
-    width: 100%;
-    border-collapse: collapse;
-}
-
-/* safari gives forms a big default margin, to vex me */
-.bar form {
-    margin: 0px;
-}
-
-.bar {
-    background-color: #dee7ec;
-    border: 1px solid #8cacbb;
-    font-size: smaller;
-    padding-top: 0px;
-    padding-bottom: 0px;
-}
-
-/* revision navigation buttons (the ones at the top right) */
-.navbuttons {
-    padding: 2px;
-    bottom: 0px;
-    text-align: right;
-    width: auto;
-    border-left: 1px dotted #8cacbb;
-    padding-left: 0.5em;
-    padding-right: 0.5em;
-}
-
-.navbuttons a {
-    font-weight: normal;
-    margin-left: 2px;
-    margin-right: 2px;
-}
-
-.navbuttons a:hover {
-    padding: 0;
-    color: blue;
-}
-
-/* navigation button in the pages (the ones at the top left, and bottom) */
-.buttons {
-}
-
-.buttons a,.buttons .search {
-    font-weight: normal;
-    color: #000088;
-    font-size: 12pt;
-    padding-left: 10px;
-    padding-right: 7px;
-    border-right: 1px dotted #8cacbb;
-}
-
-.buttons a:hover,.rbuttons a:hover {
-    font-weight: normal;
-    color: blue;
-}
-
-.feed img {
-    vertical-align: text-top;
-}
-
-.rbuttons a {
-    font-weight: normal;
-    color: #000088;
-    font-size: 12pt;
-    padding-right: 7px;
-    padding-left: 10px;
-    border-left: 1px dotted #8cacbb;
-    text-align: right;
-    width: auto;
-}
-
-
-/* ----- revision display in various places ----- */
-
-.revision .revno {
-    font-weight: bold;
-}
-
-.revision .revid {
-    font-family: monospace;
-    font-size: smaller;
-    padding-left: 1em;
-}
-
-.revision .log {
-    font-size: smaller;
-}
-
-
-/* ---- sub-links underneath the headline on some pages ----- */
-
-div.links {
-    padding-top: 0.5em;
-    padding-left: 3em;
-    font-size: 50%;
-}
-
-div.links div {
-    margin-bottom: 3px;
-}
-
-
-/* ----- list of merge points for a revision ----- */
-
-.info-entry {
-    font-size: smaller;
-    margin-bottom: 2em;
-    padding: 0px;
-    background-color: #f4f5f8;
-    border: 1px solid #8cacbb;
-    width: 60%;
-}
-
-.info-entry th {
-    background-color: #dee7ec;
-    font-weight: normal;
-    color: black;
-    text-align: right;
-    vertical-align: top;
-
-    padding-left: 0.5em;
-    padding-right: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.info-entry td {
-    font-weight: normal;
-    padding-left: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-
-/* ----- changelog page ----- */
-
-/* secret css-jutsu to make hidden margins vanish on a table */
-.log-entries {
-    border-collapse: collapse;
-    width: 98%;
-    margin-top: 0.5em;
-    margin-left: 1em;
-}
-
-.log-entries .log-header {
-    background-color: #8cacbb;
-    padding-top: 3px;
-    padding-bottom: 3px;
-    text-align: left;
-}
-
-.log-entries .revision-header {
-    font-size: 1.0em;
-}
-
-.log-entries .revision-header td {
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.log-entries .revision-header td.expand-button {
-    width: 20px;
-}
-
-.log-entries .revision-number {
-    padding-left: 0.5em;
-}
-
-.log-entries .revision-header td.date {
-    font-size: smaller;
-}
-
-.log-entries .revision-header td.author {
-    font-size: smaller;
-    padding-right: 0.5em;
-}
-
-.log-entries .revision-header .inventory-link {
-    font-size: smaller;
-    vertical-align: middle;
-    text-align: left;
-    padding-right: 0.5em;
-}
-
-.log-entries table.revision-details {
-    font-size: smaller;
-    margin-bottom: 1.0em;
-}
-
-.log-entries table.revision-details th {
-    text-align: right;
-    padding-right: 0.5em;
-    vertical-align: top;
-}
-
-.log-entries table.revision-details td.files .filename {
-    font-family: monospace;
-}
-
-.log-entries .revision-details td.files .jump {
-    font-size: smaller;
-}
-
-.log-entries .revision-details td.description {
-    font-family: monospace;
-}
-
-/* default state: hide all */
-.revision-header .hide-button { display: none; }
-.hidden-details { display: none; }
-.changelog .collapse-cl-hideall { display: none; }
-
-
-/* ----- revision page ----- */
-
-.revision-info {
-    border: none;
-    font-size: smaller;
-    margin-bottom: 1em;
-    padding: 0;
-    background-color: #f4f5f8;
-    border: 1px solid #8cacbb;
-    width: 80%;
-}
-
-/* secret css-jutsu to make hidden margins vanish on a table */
-.revision-info table {
-    border-collapse: collapse;
-    width: 100%;
-}
-
-.revision-info th {
-    background-color: #dee7ec;
-    color: black;
-    font-weight: normal;
-    text-align: right;
-    vertical-align: top;
-    
-    padding-left: 0.5em;
-    padding-right: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.revision-info td {
-    font-weight: normal;
-    padding-left: 0.5em;
-    padding-top: 2px;
-    padding-bottom: 2px;
-}
-
-.revision-info table td.description {
-    font-family: monospace;
-}
-
-.revision-info table td.files .filename {
-    font-family: monospace;
-}
-
-.revision-info table td.files a.jump {
-    font-size: smaller;
-}
-    
-.revision-info table tr.divider {
-    border-top: 1px solid #8cacbb;
-}
-
-
-/* ----- overly complex diff display ----- */
-
-.diff {
-    margin-bottom: 1em;
-    padding-top: 10px;
-    width: 100%;
-    background-color: #fcfcfc;
-    font-family: monospace;
-}
-
-
-.diff-block {
-    border-collapse: collapse;
-    width: 100%;
-    margin-bottom: 1em;
-}
-
-.diff-block th.filename {
-    font-family: monospace;
-    font-weight: normal;
-    font-size: smaller;
-    padding-left: 1em;
-    padding-top: 1em;
-    padding-bottom: 1em;
-    text-align: left;
-    background-color: #dee7ec;
-}
-
-.diff-block .diff-chunk {
-    font-size: 66%;
-    margin-top: 0px;
-    margin-bottom: 2em;
-}
-
-.diff-block th.lineno {
-    font-weight: bold;
-    border-bottom: 1px solid #8cacbb;
-    border-top: 1px solid #dee7ec;
-}
-
-.diff-block .diff-spacing {
-    line-height: 1em;
-}
-
-.diff-option-buttons {
-    margin-top: 1.0em;
-    margin-bottom: 1.0em;
-    border-collapse: collapse;
-}
-.diff-option-buttons td {
-    line-height: 1em;
-    height: 1em;
-    vertical-align: middle;
-    padding: 0px;
-}
-.diff-option-buttons .label {
-    font-size: smaller;
-    padding-left: 0.75em;
-    padding-right: 2em;
-    line-height: 1em;
-}
-.diff-option-buttons .spacey {
-    padding-right: 5em;
-}
-
-.diff-option-buttons .collapse-file-showall { display: none; }
-
-.diff-block .show-button { display: none; }
-.collapse-style-unified-content { display: none; }
-.collapse-style-sbs-show { display: none; }
-
-.diff-insert {
-    background-color: #9cc8ff;
-}
-.diff-delete {
-    background-color: #ffa0a0;
-}
-.diff-chunk td.text {
-    padding-left: 1em;
-}
-
-.diff-key {
-    display: inline;
-    font-size: smaller;
-}
-
-.diff-key-block {
-    width: 1em;
-    height: 1em;
-    border: 1px solid #8cacbb;
-}
-
-.diff-key .label {
-    padding-right: 2em;
-}
-
-.diff .lineno {
-    color: purple;
-    background-color: #dee7ec; 
-    width: 4em;
-    text-align: right;
-    padding-right: 3px;
-    border-bottom: 1px solid #8cacbb;
-}
-.diff .lineno-skip {
-    width: 4em;
-    background-color: #dee7ec;
-}
-.diff .info {
-    color: blue;
-    background-color: #dee7ec;
-    margin-top: 15px;
-}
-/*
-.diff-context-skip {
-    background-color: #dee7ec;
-}*/
-
-
-/* ----- file inventory ----- */
-
-.inventory-path {
-    margin-top: 2em;
-}
-
-table.inventory {
-    border-collapse: collapse;
-    width: 100%;
-    font-size: smaller;
-    margin-top: 1em;
-    border: 1px solid #8cacbb;
-}
-
-table.inventory td {
-    padding-top: 3px;
-    padding-bottom: 3px;
-    vertical-align: middle;
-}
-
-table.inventory th {
-    background-color: #8cacbb;
-    padding-top: 3px;
-    padding-bottom: 3px;
-    text-align: left;
-}
-
-table.inventory th.permissions {
-    padding-left: 0.5em;
-}
-
-.parity0 { background-color: #dee7ec; }
-.parity1 { background-color: #eff8fd; }
-
-.inventory td.permissions {
-    font-family: monospace;
-    padding-left: 0.5em;
-    width: 9em;
-}
-
-.inventory td.filename {
-    font-family: monospace;
-}
-
-.inventory td.size {
-    font-family: monospace;
-    font-size: smaller;
-    width: 5em;
-    line-height: 1em;
-    vertical-align: bottom;
-}
-
-.inventory td.revision {
-    padding-right: 1em;
-    text-align: left;
-    width: 8em;
-}
-
-.inventory td.date {
-    padding-right: 1em;
-    width: 11em;
-}
-
-.inventory td.changes-link {
-    font-size: smaller;
-    width: 6em;
-}
-
-.inventory td.download-link {
-    font-size: smaller;
-    width: 8em;
-}
-
-span.folder {
-    font-family: monospace;
-}
-
-
-/* ----- file annotation ----- */
-
-.annotate-path {
-    font-family: monospace;
-}
-
-.annotate table {
-    border-collapse: collapse;
-    width: 100%;
-    font-size: 66%;
-}
-
-.annotate td {
-    padding-top: 2px;
-    padding-bottom: 2px;
-    vertical-align: bottom;
-}
-
-.annotate td.lineno {
-    font-family: monospace;
-    text-align: right;
-    width: 4em;
-    padding-right: 5px;
-    font-weight: bold;
-    border-bottom: 1px solid #8cacbb;
-}
-
-.annotate td.revno {
-    font-family: monospace;
-    text-align: right;
-    width: 8em;
-    padding-right: 5px;
-    border-right: 1px solid #8cacbb;
-}
-
-.annotate td.text {
-    font-family: monospace;
-    padding-left: 1em;
-}
-
-.annotate th {
-    background-color: #8cacbb;
-    padding-top: 3px;
-    padding-bottom: 3px;
-    text-align: left;
-}
-
-.annotate th.lineno {
-    text-align: right;
-}
-
-.annotate th.revision {
-    text-align: right;
-    padding-right: 5px;
-}
-
-.annotate th.text {
-    padding-left: 1em;
-}
-
-
-/* ----- top-level browse page ----- */
-
-.browse-project-name {
-    background-color: #dee7ec;
-    padding-top: 5px;
-    padding-bottom: 5px;
-    margin-left: 1em;
-    margin-right: 1em;
-    padding-left: 1em;
-    width: 90%;
-    font-weight: bold;
-}
-
-.browse-project-description {
-	padding-top: 1em;
-	padding-bottom: 1em;
-	margin-left: 2em;
-    width: 90%;
-}
-	
-.browse-view {
-	margin-top: 1em;
-	margin-bottom: 1em;
-	margin-left: 2em;
-}
-
-.browse-view table {
-    border-collapse: collapse;
-    width: 90%;
-    margin-bottom: 2em;
-}
-
-.browse-view table tr.heading {
-    background-color: #8cacbb;
-    border-bottom: solid 1px #8cacbb;
-    font-size: smaller;
-}
-
-.browse-view td,.browse-view th {
-    padding: 0.25em 0.5em 0.25em 0.5em;
-}
-
-.browse-view td.name {
-    width: 20%;
-}
-
-.browse-view td.last-update {
-    font-size: smaller;
-    width: 20em;
-}
-
-.browse-view td.url {
-    font-family: monospace;
-    font-size: smaller;
-    padding-left: 3em;
-    padding-top: 0px;
-}
-
-.browse-view td.feed {
-	width: 16px;
-}

=== added file 'loggerhead/static/css/zptstyle.css'
--- loggerhead/static/css/zptstyle.css	1970-01-01 00:00:00 +0000
+++ loggerhead/static/css/zptstyle.css	2008-06-11 03:53:53 +0000
@@ -0,0 +1,670 @@
+
+/* from launchpad */
+body {
+    background-color: #fcfcfc;
+    color: black;
+    font-family: "Lucida Grande", "Trebuchet MS", sans-serif;
+    font-size: 12pt;
+    padding: 0px;
+    text-align: left;
+}
+
+a {
+    text-decoration: none;
+    color: #000088;
+}
+a:hover {
+    color: black;
+}
+
+img {
+    border: none;
+}
+
+h1 {
+    color: #708a96;
+    font-weight: normal;
+    font-size: 1.6em;
+    margin-bottom: 0em;
+}
+h1 .filename {
+    font-family: monospace;
+}
+
+.branch-name {
+    color: #47626e;
+}
+
+.footer {
+    margin-top: 1em;
+    margin-bottom: 1em;
+}
+.footer .banner {
+    margin-right: 5px;
+}
+.footer .banner img {
+    border: 1px solid #8cacbb;
+}
+
+.collapse-triangle {
+    padding-right: 0.5em;
+}
+
+.hide-all {
+    font-size: smaller;
+    padding: 0.25em 0.75em 0.25em 0.75em;
+}
+
+
+
+/* ----- navigation bar at the top ----- */
+
+.navbar {
+}
+
+.bar table {
+    width: 100%;
+    border-collapse: collapse;
+}
+
+/* safari gives forms a big default margin, to vex me */
+.bar form {
+    margin: 0px;
+}
+
+.bar {
+    background-color: #dee7ec;
+    border: 1px solid #8cacbb;
+    font-size: smaller;
+    padding-top: 0px;
+    padding-bottom: 0px;
+}
+
+/* revision navigation buttons (the ones at the top right) */
+.navbuttons {
+    padding: 2px;
+    bottom: 0px;
+    text-align: right;
+    width: auto;
+    border-left: 1px dotted #8cacbb;
+    padding-left: 0.5em;
+    padding-right: 0.5em;
+}
+
+.navbuttons a {
+    font-weight: normal;
+    margin-left: 2px;
+    margin-right: 2px;
+}
+
+.navbuttons a:hover {
+    padding: 0;
+    color: blue;
+}
+
+/* navigation button in the pages (the ones at the top left, and bottom) */
+.buttons {
+}
+
+.buttons a,.buttons .search {
+    font-weight: normal;
+    color: #000088;
+    font-size: 12pt;
+    padding-left: 10px;
+    padding-right: 7px;
+    border-right: 1px dotted #8cacbb;
+}
+
+.buttons a:hover,.rbuttons a:hover {
+    font-weight: normal;
+    color: blue;
+}
+
+.feed img {
+    vertical-align: text-top;
+}
+
+.rbuttons a {
+    font-weight: normal;
+    color: #000088;
+    font-size: 12pt;
+    padding-right: 7px;
+    padding-left: 10px;
+    border-left: 1px dotted #8cacbb;
+    text-align: right;
+    width: auto;
+}
+
+
+/* ----- revision display in various places ----- */
+
+.revision .revno {
+    font-weight: bold;
+}
+
+.revision .revid {
+    font-family: monospace;
+    font-size: smaller;
+    padding-left: 1em;
+}
+
+.revision .log {
+    font-size: smaller;
+}
+
+
+/* ---- sub-links underneath the headline on some pages ----- */
+
+div.links {
+    color: #708a96;
+    padding-top: 0.5em;
+    padding-left: 3em;
+    font-size: 0.8em;
+    margin-bottom: 2em;
+}
+
+div.links div {
+    margin-bottom: 3px;
+}
+
+
+/* ----- list of merge points for a revision ----- */
+
+.info-entry {
+    font-size: smaller;
+    margin-bottom: 2em;
+    padding: 0px;
+    background-color: #f4f5f8;
+    border: 1px solid #8cacbb;
+    width: 60%;
+}
+
+.info-entry th {
+    background-color: #dee7ec;
+    font-weight: normal;
+    color: black;
+    text-align: right;
+    vertical-align: top;
+
+    padding-left: 0.5em;
+    padding-right: 0.5em;
+    padding-top: 2px;
+    padding-bottom: 2px;
+}
+
+.info-entry td {
+    font-weight: normal;
+    padding-left: 0.5em;
+    padding-top: 2px;
+    padding-bottom: 2px;
+}
+
+
+/* ----- changelog page ----- */
+
+/* secret css-jutsu to make hidden margins vanish on a table */
+.log-entries {
+    border-collapse: collapse;
+    width: 98%;
+    margin-top: 0.5em;
+    margin-left: 1em;
+    font-size: 1em;
+}
+
+.log-entries .log-header {
+    background-color: #8cacbb;
+    padding-top: 3px;
+    padding-bottom: 3px;
+    text-align: left;
+}
+
+.log-entries .revision-header {
+    font-size: normal;
+}
+
+.log-entries .revision-header td {
+    padding-top: 2px;
+    padding-bottom: 2px;
+}
+
+.log-entries .revision-header td.expand-button {
+    width: 20px;
+}
+
+.log-entries .revision-number {
+    padding-left: 0.5em;
+}
+
+.log-entries .revision-header td.date {
+    font-size: smaller;
+}
+
+.log-entries .revision-header td.author {
+    font-size: smaller;
+    padding-right: 0.5em;
+}
+
+.log-entries .revision-header .inventory-link {
+    font-size: smaller;
+    vertical-align: middle;
+    text-align: left;
+    padding-right: 0.5em;
+}
+
+.log-entries table.revision-details {
+    font-size: smaller;
+    margin-bottom: 1.0em;
+}
+
+.log-entries table.revision-details th {
+    text-align: right;
+    padding-right: 0.5em;
+    vertical-align: top;
+}
+
+.log-entries table.revision-details td.files .filename {
+    font-family: monospace;
+}
+
+td.files {
+    padding-top: 0em;
+    padding-bottom: 0em;
+}
+th.files {
+    padding-top: 0em;
+    padding-bottom: 0em;
+}
+
+.log-entries .revision-details td.files .jump {
+    font-size: smaller;
+}
+
+.log-entries .revision-details td.description {
+    font-family: monospace;
+}
+
+/* default state: hide all */
+.revision-header .hide-button { display: none; }
+.hidden-details { display: none; }
+.changelog .collapse-cl-hideall { display: none; }
+
+
+/* ----- revision page ----- */
+
+.revision-info {
+    border: none;
+    margin-bottom: 1em;
+    padding: 0;
+    background-color: #f4f5f8;
+    border: 1px solid #8cacbb;
+    width: 80%;
+}
+
+/* secret css-jutsu to make hidden margins vanish on a table */
+.revision-info table {
+    border-collapse: collapse;
+    width: 100%;
+}
+
+.revision-info th {
+    background-color: #dee7ec;
+    color: black;
+    font-weight: normal;
+    text-align: right;
+    vertical-align: top;
+    font-size: 0.8em;
+    padding-left: 0.5em;
+    padding-right: 0.5em;
+    padding-top: 2px;
+    padding-bottom: 2px;
+}
+
+.revision-info td {
+    font-weight: normal;
+    font-size: 0.8em;
+    padding-left: 0.5em;
+    padding-top: 2px;
+    padding-bottom: 2px;
+}
+
+.revision-info table td.date {
+    font-family: monospace;
+    font-size: 1em;
+}
+
+.revision-info table td.description {
+    font-family: monospace;
+    font-size: 1em;
+}
+
+.revision-info table td.files {
+    font-family: monospace;
+    font-size: 1em;
+}
+
+.revision-info table td.files a.jump {
+    font-size: smaller;
+}
+    
+.revision-info table tr.divider {
+    border-top: 1px solid #8cacbb;
+}
+
+
+/* ----- overly complex diff display ----- */
+
+.diff {
+    margin-bottom: 1em;
+    padding-top: 10px;
+    width: 100%;
+    background-color: #fcfcfc;
+    font-family: monospace;
+}
+
+
+.diff-block {
+    border-collapse: collapse;
+    width: 100%;
+    margin-bottom: 1em;
+}
+
+.diff-block th.filename {
+    font-family: monospace;
+    font-weight: normal;
+    font-size: 1em;
+    padding-left: 1em;
+    padding-top: 1em;
+    padding-bottom: 1em;
+    text-align: left;
+    background-color: #dee7ec;
+}
+
+.diff-block .diff-chunk {
+    font-size: 0.9em;
+    margin-top: 0px;
+    margin-bottom: 2em;
+}
+
+.diff-block th.lineno {
+    font-weight: bold;
+    border-bottom: 1px solid #8cacbb;
+    border-top: 1px solid #dee7ec;
+}
+
+.diff-block .diff-spacing {
+    line-height: 1em;
+}
+
+.diff-option-buttons {
+    margin-top: 1.0em;
+    margin-bottom: 1.0em;
+    border-collapse: collapse;
+}
+.diff-option-buttons td {
+    line-height: 1em;
+    height: 1em;
+    vertical-align: middle;
+    padding: 0px;
+}
+.diff-option-buttons .label {
+    font-size: smaller;
+    padding-left: 0.75em;
+    padding-right: 2em;
+    line-height: 1em;
+}
+.diff-option-buttons .spacey {
+    padding-right: 5em;
+}
+
+.diff-option-buttons .collapse-file-showall { display: none; }
+
+.diff-block .show-button { display: none; }
+.collapse-style-unified-content { display: none; }
+.collapse-style-sbs-show { display: none; }
+
+.diff-insert {
+    background-color: #9cc8ff;
+}
+.diff-delete {
+    background-color: #ffa0a0;
+}
+.diff-chunk td.text {
+    padding-left: 1em;
+}
+
+.diff-key {
+    display: inline;
+    font-size: smaller;
+}
+
+.diff-key-block {
+    width: 1em;
+    height: 1em;
+    border: 1px solid #8cacbb;
+}
+
+.diff-key .label {
+    padding-right: 2em;
+}
+
+.diff .lineno {
+    color: purple;
+    background-color: #dee7ec; 
+    width: 4em;
+    text-align: right;
+    padding-right: 3px;
+    border-bottom: 1px solid #8cacbb;
+}
+.diff .lineno-skip {
+    width: 4em;
+    background-color: #dee7ec;
+}
+.diff .info {
+    color: blue;
+    background-color: #dee7ec;
+    margin-top: 15px;
+}
+/*
+.diff-context-skip {
+    background-color: #dee7ec;
+}*/
+
+
+/* ----- file inventory ----- */
+
+.inventory-path {
+    margin-top: 2em;
+}
+
+table.inventory {
+    border-collapse: collapse;
+    width: 100%;
+    font-size: smaller;
+    margin-top: 1em;
+    border: 1px solid #8cacbb;
+}
+
+table.inventory td {
+    padding-top: 3px;
+    padding-bottom: 3px;
+    vertical-align: middle;
+}
+
+table.inventory th {
+    background-color: #8cacbb;
+    padding-top: 3px;
+    padding-bottom: 3px;
+    text-align: left;
+}
+
+table.inventory th.permissions {
+    padding-left: 0.5em;
+}
+
+.parity0 { background-color: #dee7ec; }
+.parity1 { background-color: #eff8fd; }
+
+.inventory td.permissions {
+    font-family: monospace;
+    padding-left: 0.5em;
+    width: 9em;
+}
+
+.inventory td.filename {
+    font-family: monospace;
+}
+
+.inventory td.size {
+    font-family: monospace;
+    font-size: smaller;
+    width: 5em;
+}
+
+.inventory td.revision {
+    padding-right: 1em;
+    text-align: left;
+    width: 8em;
+}
+
+.inventory td.date {
+    padding-right: 1em;
+    width: 11em;
+}
+
+.inventory td.changes-link {
+    font-size: smaller;
+    width: 6em;
+}
+
+.inventory td.download-link {
+    font-size: smaller;
+    width: 8em;
+}
+
+span.folder {
+    font-family: monospace;
+}
+
+
+/* ----- file annotation ----- */
+
+.annotate-path {
+    font-family: monospace;
+}
+
+.annotate table {
+    border-collapse: collapse;
+    width: 100%;
+    font-size: 66%;
+}
+
+.annotate td {
+    padding-top: 2px;
+    padding-bottom: 2px;
+    vertical-align: bottom;
+}
+
+.annotate td.lineno {
+    font-family: monospace;
+    text-align: right;
+    width: 4em;
+    padding-right: 5px;
+    font-weight: bold;
+    border-bottom: 1px solid #8cacbb;
+}
+
+.annotate td.revno {
+    font-family: monospace;
+    text-align: right;
+    width: 8em;
+    padding-right: 5px;
+    border-right: 1px solid #8cacbb;
+}
+
+.annotate td.text {
+    font-family: monospace;
+    padding-left: 1em;
+}
+
+.annotate th {
+    background-color: #8cacbb;
+    padding-top: 3px;
+    padding-bottom: 3px;
+    text-align: left;
+}
+
+.annotate th.lineno {
+    text-align: right;
+}
+
+.annotate th.revision {
+    text-align: right;
+    padding-right: 5px;
+}
+
+.annotate th.text {
+    padding-left: 1em;
+}
+
+
+/* ----- top-level browse page ----- */
+
+.browse-project-name {
+    background-color: #dee7ec;
+    padding-top: 5px;
+    padding-bottom: 5px;
+    margin-left: 1em;
+    margin-right: 1em;
+    padding-left: 1em;
+    width: 90%;
+    font-weight: bold;
+}
+
+.browse-project-description {
+	padding-top: 1em;
+	padding-bottom: 1em;
+	margin-left: 2em;
+    width: 90%;
+}
+	
+.browse-view {
+	margin-top: 1em;
+	margin-bottom: 1em;
+	margin-left: 2em;
+}
+
+.browse-view table {
+    border-collapse: collapse;
+    width: 90%;
+    margin-bottom: 2em;
+}
+
+.browse-view table tr.heading {
+    background-color: #8cacbb;
+    border-bottom: solid 1px #8cacbb;
+    font-size: smaller;
+}
+
+.browse-view td,.browse-view th {
+    padding: 0.25em 0.5em 0.25em 0.5em;
+}
+
+.browse-view td.name {
+    width: 20%;
+}
+
+.browse-view td.last-update {
+    font-size: smaller;
+    width: 20em;
+}
+
+.browse-view td.url {
+    font-family: monospace;
+    font-size: smaller;
+    padding-left: 3em;
+    padding-top: 0px;
+}
+
+.browse-view td.feed {
+	width: 16px;
+}

=== modified file 'loggerhead/static/javascript/collapse.js'
--- loggerhead/static/javascript/collapse.js	2007-01-14 01:24:15 +0000
+++ loggerhead/static/javascript/collapse.js	2008-06-11 02:26:38 +0000
@@ -102,3 +102,25 @@
     console.log("[" + ds + "] " + s);
 }
 
+function diff_url(url) {
+    if (document.cookie.indexOf('diff=unified') >= 0) {
+        this.location.href = url + "-u";
+    } else {
+        this.location.href = url + "-s";
+    }
+}
+
+function show_sbs() {
+	collapseDisplay('style', 'sbs', 'table');
+	collapseDisplay('style', 'unified', 'none');
+	document.cookie='diff=sbs';
+}
+function show_unified() {
+	collapseDisplay('style', 'unified', 'table');
+	collapseDisplay('style', 'sbs', 'none');
+	document.cookie='diff=unified';
+}
+function load() {
+	sortCollapseElements();
+	if (document.cookie.indexOf('diff=unified') >= 0) { show_unified(); }
+}

=== added file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	1970-01-01 00:00:00 +0000
+++ loggerhead/templatefunctions.py	2008-06-11 00:02:50 +0000
@@ -0,0 +1,62 @@
+import os
+import turbogears
+from turbosimpletal.zptsupport import zpt
+
+templatefunctions = {}
+def templatefunc(func):
+    templatefunctions[func.__name__] = func
+    return func
+
+
+_base = os.path.dirname(__file__)
+def _pt(name):
+    return zpt(os.path.join(_base, 'templates', name + '.pt'))
+
+
+templatefunctions['macros'] = _pt('macros').macros
+
+@templatefunc
+def file_change_summary(url, entry, modified_file_link):
+    return _pt('revisionfilechanges').expand(
+        url=url, entry=entry, modified_file_link=modified_file_link,
+        **templatefunctions)
+
+@templatefunc
+def revisioninfo(url, branch, entry, modified_file_link=None):
+    from loggerhead import util
+    return _pt('revisioninfo').expand(
+        url=url, change=entry, branch=branch, util=util,
+        modified_file_link=modified_file_link,
+        **templatefunctions)
+
+@templatefunc
+def collapse_button(group, name, normal='block'):
+    return _pt('collapse-button').expand(
+        group=group, name=name, normal=normal, tg=turbogears,
+        **templatefunctions)
+
+@templatefunc
+def collapse_all_button(group, normal='block'):
+    return _pt('collapse-all-button').expand(
+        group=group, normal=normal, tg=turbogears,
+        **templatefunctions)
+
+@templatefunc
+def revno_with_nick(entry):
+    if entry.branch_nick:
+        extra = ' ' + entry.branch_nick
+    else:
+        extra = ''
+    return '(%s%s)'%(entry.revno, extra)
+
+@templatefunc
+def modified_file_link_rev(url, entry, item):
+    return _pt('modified-file-link-rev').expand(
+        url=url, entry=entry, item=item,
+        **templatefunctions)
+
+@templatefunc
+def modified_file_link_log(url, entry, item):
+    return _pt('modified-file-link-log').expand(
+        url=url, entry=entry, item=item,
+        **templatefunctions)

=== removed file 'loggerhead/templates/annotate.kid'
--- loggerhead/templates/annotate.kid	2008-03-10 04:27:19 +0000
+++ loggerhead/templates/annotate.kid	1970-01-01 00:00:00 +0000
@@ -1,42 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
-    py:extends="'master.kid'">
-<head>
-    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" py:replace="''"/>
-    <title> ${branch.friendly_name} : contents of ${path} at revision ${change.revno} </title>
-</head>
-
-<body>
-
-${navbar()}
-
-<h1> <span class="branch-name">${branch.friendly_name}</span> : <span class="annotate-path">${path}</span> (revision ${change.revno})
-	<div class="links">
-	    <div> <b>&#8594;</b> <a href="${branch.url([ '/files', revid ], **util.get_context(clear=1))}"> browse files </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.url('/revision', **util.get_context(clear=1, start_revid=revid))}"> view revision </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.url('/changes', **util.get_context(clear=1, start_revid=revid, filter_file_id=file_id))}"> view changes to this file </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.url([ '/download', revid, file_id, filename ])}"> download file </a> </div>
-	</div>
-</h1>
-
-<div class="annotate">
-    <table>
-        <tr>
-            <th class="lineno"> Line# </th>
-            <th class="revision"> Revision </th>
-            <th class="text"> Contents </th>
-        </tr>
-
-        <tr py:for="line in contents" class="parity${line.parity}">
-            <td class="lineno ${line.status}"><a name="L${line.lineno}"> ${line.lineno} </a></td>
-            <td class="revno ${line.status}">
-                <a py:if="line.status=='changed'" href="${branch.url('/revision', **util.get_context(clear=1, start_revid=line.change.revid, filter_file_id=file_id))}"
-                    title="${line.change.revno} by ${util.hide_email(line.change.author)}, ${util.date_gtime(line.change.date)}})">${util.trunc(line.change.revno)}</a>
-            </td>
-            <td class="text ${line.status}"> ${line.text} </td>
-        </tr>
-    </table>
-</div>
-
-</body>
-</html>

=== added file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/annotate.pt	2008-05-29 04:43:39 +0000
@@ -0,0 +1,88 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
+          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+  <head>
+    <title metal:fill-slot="title"
+           tal:content="string:${branch/friendly_name} : contents of ${path}
+                        at revision ${change/revno}">
+      ${branch.friendly_name} : contents of ${path} at revision ${change.revno}
+    </title>
+  </head>
+
+  <body>
+
+    <h1 metal:fill-slot="heading">
+      <span class="branch-name" tal:content="branch/friendly_name">
+        ${branch.friendly_name}
+      </span>
+      :
+      <span class="annotate-path" tal:content="path">
+        ${path}
+      </span>
+      (revision <tal:revno content="change/revno">${change.revno}</tal:revno>)
+    </h1>
+
+    <div metal:fill-slot="content">
+      <div class="links">
+        <div>
+          <b>&#8594;</b>
+          <a tal:attributes="href python:url(['/files', revid], clear=1)">
+            browse files
+          </a>
+        </div>
+        <div>
+          <b>&#8594;</b>
+          <a tal:attributes="href python:url(['/revision'], clear=1, start_revid=revid)">
+            view revision
+          </a>
+        </div>
+        <div>
+          <b>&#8594;</b>
+          <a tal:attributes="href python:url(['/changes'], clear=1, start_revid=revid, filter_file_id=file_id)">
+            view changes to this file
+          </a>
+        </div>
+        <div>
+          <b>&#8594;</b>
+          <a tal:attributes="href python:url(['/download', revid, file_id, filename])">
+            download file
+          </a>
+        </div>
+      </div>
+
+      <div class="annotate">
+        <table>
+          <tr>
+            <th class="lineno"> Line# </th>
+            <th class="revision"> Revision </th>
+            <th class="text"> Contents </th>
+          </tr>
+
+          <tr tal:repeat="line contents"
+              tal:attributes="class string:parity${line/parity}">
+            <td tal:attributes="class string:lineno ${line/status}">
+              <a tal:attributes="name string:L${line/lineno}"
+                 tal:content="line/lineno">
+                ${line.lineno}
+              </a>
+            </td>
+            <td tal:attributes="class string:revno ${line/status}">
+              <a tal:condition="python:line.status=='changed'"
+                 tal:content="python:util.trunc(line.change.revno)"
+                 tal:attributes="href python:url(['/revision'], clear=1, start_revid=line.change.revid, filter_file_id=file_id);
+                                 title python:'%s by %s, on %s (%s)'%(line.change.revno, util.hide_email(line.change.author), line.change.date.strftime('%d %b %Y %H:%M'), util.date_time(line.change.date))">
+
+                ${util.trunc(line.change.revno)}
+              </a>
+            </td>
+            <td tal:attributes="class string:text ${line/status}"
+                tal:content="line/text">
+              ${line.text}
+            </td>
+          </tr>
+        </table>
+      </div>
+
+    </div>
+  </body>
+</html>

=== removed file 'loggerhead/templates/atom.kid'
--- loggerhead/templates/atom.kid	2007-03-29 06:55:55 +0000
+++ loggerhead/templates/atom.kid	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-<?xml version="1.0" encoding="utf-8"?>
-<feed xmlns="http://www.w3.org/2005/Atom" xmlns:py="http://purl.org/kid/ns#">
-    <title> bazaar changes for ${branch.friendly_name} </title>
-    <updated>${updated}</updated>
-    <id>${branch.url(['atom'])}</id>
-    <link rel="self" href="${branch.url([''])}" />
-
-	<entry py:for="entry in changes">
-	    <title> ${entry.revno}: ${entry.short_comment} </title>
-	    <updated>${entry.date.isoformat() + 'Z'}</updated>
-	    <id>${branch.url(['revision', entry.revid ])}</id>
-	    <author> <name> ${util.hide_email(entry.author)} </name> </author>
-	    <content type="text">
-            ${entry.comment}
-	    </content>
-	    <link rel="alternate"
-                  href="${branch.url(['revision', entry.revid ])}" />
-	</entry>
-</feed>

=== added file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/atom.pt	2008-06-10 04:48:26 +0000
@@ -0,0 +1,31 @@
+<?xml version="1.0" encoding="utf-8"?>
+<feed xmlns="http://www.w3.org/2005/Atom" xmlns:tal="http://xml.zope.org/namespaces/tal">
+  <title>
+    bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
+  </title>
+  <updated tal:content="updated">${updated}</updated>
+  <id tal:content="python:branch.url(['atom'])">url</id>
+  <link rel="self" tal:attributes="href python:branch.url([''])" />
+
+  <entry tal:repeat="entry changes">
+    <title tal:content="string: ${entry/revno}: ${entry/short_comment}">
+      ${entry/revno}: ${entry/short_comment}
+    </title>
+    <updated tal:content="entry/date/isoformat">
+    Z
+    </updated>
+    <id tal:content="python:branch.url(['revision', entry.revid])">
+      ID
+    </id>
+    <author>
+      <name tal:content="python:util.hide_email(entry.author)">
+        author
+      </name>
+    </author>
+    <content type="text" tal:content="entry/comment">
+      comment
+    </content>
+    <link rel="alternate"
+          tal:attributes="href python:branch.url(['revision', entry.revid])" />
+  </entry>
+</feed>

=== removed file 'loggerhead/templates/browse.kid'
--- loggerhead/templates/browse.kid	2008-03-09 00:22:47 +0000
+++ loggerhead/templates/browse.kid	1970-01-01 00:00:00 +0000
@@ -1,55 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
-    py:extends="'master.kid'">
-<head>
-    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" py:replace="''"/>
-    <title> loggerhead branches </title>
-</head>
-
-<body>
-
-<h1>
-    <span py:if="title">${title}</span>
-    <span py:if="not title"> bazaar branches in loggerhead </span>
-</h1>
-
-<div class="browse-project" py:for="project in projects">
-    <div class="browse-project-name">
-        ${project.friendly_name}
-    </div>
-    
-    <div class="browse-project-description" py:if="len(project.long_description) > 0">
-    ${XML(project.long_description)}
-    </div>
-    
-    <div class="browse-view">
-        <table>
-        <tr class="heading">
-            <th> Name </th>
-            <th> Description </th>
-            <th> Last change </th>
-            <th />
-        </tr>
-        <span py:for="view in project.views">
-            <tr>
-            <td class="name"> <a href="${tg.url([ '/' + project.name, view.name ])}">${view.friendly_name}</a> </td>
-            <td class="description"> ${view.description} </td>
-            <td class="last-update"> ${view.get_history().has_revisions and util.approximatedate(view.last_updated()) or "(no revisions)"} </td>
-            <td class="feed">
-                <a href="${tg.url([ '/' + project.name, view.name, 'atom' ])}" title="atom feed">
-                    <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" alt="atom feed" />
-                </a>
-            </td>
-            </tr>
-            <tr py:if="view.branch_url is not None">
-                <td class="name"> </td>
-                <td class="description url" colspan="2"> <a href="${view.branch_url}">${view.branch_url}</a> </td>
-            </tr>
-        </span>
-        </table>
-    </div>
-
-</div>
-
-</body>
-</html>

=== added file 'loggerhead/templates/browse.pt'
--- loggerhead/templates/browse.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/browse.pt	2008-06-10 04:49:54 +0000
@@ -0,0 +1,87 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+  <head>
+    <title metal:fill-slot="title">
+      loggerhead branches
+    </title>
+  </head>
+
+  <body>
+
+    <h1 metal:fill-slot="heading" style="padding-bottom: 1em">
+      <tal:title condition="title" content="title">
+        ${title}
+      </tal:title>
+      <tal:no-title condition="not:title">
+        bazaar branches in loggerhead
+      </tal:no-title>
+    </h1>
+
+    <div metal:fill-slot="content">
+      <div class="browse-project"
+           tal:repeat="project projects">
+        <div class="browse-project-name"
+             tal:content="project/friendly_name">
+          ${project.friendly_name}
+        </div>
+
+        <div class="browse-project-description"
+             tal:condition="python:len(project.long_description) > 0"
+             tal:content="structure project/long_description">
+          ${XML(project.long_description)}
+        </div>
+
+        <div class="browse-view">
+          <table>
+            <tr class="heading">
+              <th>
+                Name
+              </th>
+              <th>
+                Description
+              </th>
+              <th>
+                Last change
+              </th>
+              <th />
+            </tr>
+            <tal:block repeat="view project/views">
+              <tr>
+                <td class="name">
+                  <a tal:attributes="href python:tg.url(['/' + project.name, view.name])"
+                     tal:content="view/friendly_name">
+                    ${view.friendly_name}
+                  </a>
+                </td>
+                <td class="description"
+                    tal:content="view/description">
+                  ${view.description}
+                </td>
+                <td class="last-update"
+                    tal:content="python:view.get_history().has_revisions and util._approximatedate(view.last_updated()) or '(no revisions)'"></td>
+                <td class="feed">
+                  <a tal:attributes="href python:tg.url(['/' + project.name, view.name, 'atom'])"
+                     title="atom feed">
+                    <img tal:attributes="src python:tg.url('/static/images/feed-icon-16x16.gif')"
+                         alt="atom feed" />
+                  </a>
+                </td>
+              </tr>
+              <tr tal:condition="python:view.branch_url is not None">
+                <td class="name">
+                </td>
+                <td class="description url" colspan="2">
+                  <a tal:attributes="href view/branch_url"
+                     tal:content="view/branch_url">
+                    ${view.branch_url}
+                  </a>
+                </td>
+              </tr>
+            </tal:block>
+          </table>
+        </div>
+
+      </div>
+    </div>
+  </body>
+</html>

=== removed file 'loggerhead/templates/changelog.kid'
--- loggerhead/templates/changelog.kid	2008-03-10 04:22:42 +0000
+++ loggerhead/templates/changelog.kid	1970-01-01 00:00:00 +0000
@@ -1,157 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
-    py:extends="'master.kid'">
-<head>
-    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" py:replace="''"/>
-    <title> ${branch.friendly_name} : changes </title>
-    <link rel="alternate" type="application/atom+xml" href="${branch.url('/atom')}" title="RSS feed for ${branch.friendly_name}" />
-    
-    <span py:def="loglink(revid, text)">
-        <a title="Show history" href="${branch.url('/changes', **util.get_context(start_revid=revid))}" class="revlink"> ${text} </a>
-    </span>
-
-    <span py:def="file_link(filename, file_id, revid)">
-        <a href="${branch.url([ '/annotate', revid ], **util.get_context(file_id=file_id))}" title="Annotate ${filename}">${filename}</a>
-    </span>
-    
-    <span py:def="file_diff_link(revid, filename)">
-        <a href="javascript:diff_url('${branch.context_url([ '/revision', revid ]) + '#' + filename}')"
-           title="View diff for ${filename}">${filename}</a>
-    </span>
-    
-    ${use_collapse_buttons()}
-
-    <script type="text/javascript"> <!--
-    function diff_url(url) {
-        if (document.cookie.indexOf('diff=unified') >= 0) {
-            this.location.href = url + "-u";
-        } else {
-            this.location.href = url + "-s";
-        }
-    }
-    // --> </script>
-</head>
-
-<body onload="javascript:sortCollapseElements();">
-
-${navbar()}
-
-<h1> <span class="branch-name">${branch.friendly_name}</span> : changes
-<span py:if="filter_file_id"> to <span class="filename">${history.get_path(revid, filter_file_id)}</span></span>
-<span py:if="viewing_from"> from ${history.get_revno(start_revid)} </span>
-<span py:if="query"> matching "${query}"</span>
-</h1>
-
-<span py:if="search_failed">
-    Sorry, no results found for your search.
-</span>
-
-<span py:if="not changes">
-    No revisions!
-</span>
-
-<span py:if="not search_failed" class="changelog"> ${collapse_all_button('cl')} </span>
-
-<table class="log-entries">
-    <tr class="log-header">
-        <th class="revision-number">Rev</th>
-        <th></th>
-        <th class="summary">Summary</th>
-        <span py:strip="True" py:if="all_same_author">
-            <th></th>
-        </span>
-        <span py:strip="True" py:if="not all_same_author">
-            <th class="author">Committer</th>
-        </span>
-        <th class="date" colspan="2">Date</th>
-    </tr>
-    <div py:for="entry in changes" class="revision">
-        <a name="entry-${entry.revno}" />
-        <tr class="revision-header parity${entry.parity}">
-            <td class="revision-number"> ${revision_link(entry.revid, util.trunc(entry.revno))} </td>
-            <td class="expand-button"> ${collapse_button('cl', entry.revno)} </td>
-            <td class="summary"> ${revision_link(entry.revid, entry.short_comment)} </td>
-            <span py:strip="True" py:if="all_same_author">
-                <td></td>
-            </span>
-            <span py:strip="True" py:if="not all_same_author">
-                <td class="author"> ${util.trunc(util.hide_email(entry.author), 20)} </td>
-            </span>
-            <td class="date"> ${util.approximatedate(entry.date)} </td>
-            <td class="inventory-link"> 
-                <a href="${branch.url([ '/files', entry.revid ])}"
-                    title="Files at revision ${entry.revno}"> files</a>
-            </td>
-        </tr>
-        
-        <tr class="revision-details-block parity${entry.parity}">
-            <td colspan="2"></td>
-            <td colspan="4"><table class="revision-details hidden-details collapse-cl-${entry.revno}-content">
-                <tr py:if="len(entry.merge_points) > 0">
-                    <th class="children"> merged in: </th>
-                    <td class="children">
-                        <span py:for="child in entry.merge_points">
-                            ${loglink(child.revid, '(' + child.revno + util.if_present(' %s', child.branch_nick) + ')')} <br />
-                        </span>
-                    </td>
-                </tr>
-                <tr py:if="len(entry.parents) > 1">
-                    <th class="parents"> merged from: </th>
-                    <td class="parents">
-                        <span py:for="parent in entry.parents[1:]">
-                            ${loglink(parent.revid, parent.revno + util.if_present(' (%s)', parent.branch_nick))} <br />
-                        </span>
-                    </td>
-                </tr>
-                <span py:strip="True" py:if="all_same_author">
-                    <tr>
-                        <th class="author">committed by:</th>
-                        <td class="author"> ${util.hide_email(entry.author)} </td>
-                    </tr>
-                </span>
-                <tr>
-                    <th class="description">description:</th>
-                    <td class="description"><span py:for="line in entry.comment_clean">${XML(line)} <br /></span> </td>
-                </tr>
-                <tr py:if="entry.changes.added">
-                    <th class="files"> files added: </th>
-                    <td class="files"> <span py:for="filename, fid in entry.changes.added" class="filename">${file_link(filename, fid, entry.revid)} <br /></span> </td>
-                </tr>
-                <tr py:if="entry.changes.removed">
-                    <th class="files"> files removed: </th>
-                    <td class="files"> <span py:for="filename, fid in entry.changes.removed" class="filename">${file_link(filename, fid, entry.revid)} <br /></span> </td>
-                </tr>
-                <tr py:if="entry.changes.renamed">
-                    <th class="files"> files renamed: </th>
-                    <td class="files"> <span py:for="old_filename, new_filename, fid in entry.changes.renamed" class="filename">
-                        ${file_link(old_filename, fid, entry.revid)} => ${file_link(new_filename, fid, entry.revid)}<br />
-                    </span> </td>
-                </tr>
-                <tr py:if="entry.changes.modified">
-                    <th class="files"> files modified: </th>
-                    <td class="files"> <span py:for="item in entry.changes.modified">
-                        <span class="filename">${file_diff_link(entry.revid, item.filename)}</span>
-                        <br />
-                    </span> </td>
-                </tr>
-                </table>
-            </td>
-        </tr>
-    </div>
-</table>
-
-<div py:if="navigation.prev_page_revid or navigation.next_page_revid" class="bar">
-    <table>
-        <tr>
-        	<td class="buttons">
-            	<a py:if="navigation.prev_page_revid" href="${navigation.prev_page_url}"> &lt;&lt; page </a>
-	 		</td>
- 			<td class="rbuttons" align="right">
-            	<a py:if="navigation.next_page_revid" href="${navigation.next_page_url}"> page &gt;&gt; </a>
- 			</td>
- 		</tr>
- 	</table>
-</div>
-
-</body>
-</html>

=== added file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/changelog.pt	2008-06-11 04:51:53 +0000
@@ -0,0 +1,192 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<tal:block define="onload string:sortCollapseElements()">
+  <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+    <head>
+      <title metal:fill-slot="title"
+             tal:content="string:${branch/friendly_name} : changes">
+        ${branch.friendly_name} : changes
+      </title>
+      <metal:block fill-slot="header_extras">
+        <link rel="alternate" type="application/atom+xml"
+              tal:attributes="href python:url(['/atom']);
+                              title string:RSS feed for ${branch/friendly_name}"
+              />
+
+        <script type="text/javascript"
+                tal:attributes="src python:tg.url('/static/javascript/collapse.js')">
+        </script>
+      </metal:block>
+    </head>
+
+    <body>
+
+      <h1 style="margin-bottom: 1em" metal:fill-slot="heading">
+        <span class="branch-name" tal:content="branch/friendly_name">
+          ${branch.friendly_name}
+        </span>
+        : changes
+        <tal:block condition="filter_file_id">
+          to
+          <span class="filename"
+                tal:content="python:history.get_path(revid, filter_file_id)">
+          </span>
+        </tal:block>
+        <tal:block condition="viewing_from">
+          from <tal:block content="python:history.get_revno(start_revid)" />
+        </tal:block>
+        <tal:block condition="query">
+          matching <tal:block content="query" />
+        </tal:block>
+      </h1>
+
+      <div metal:fill-slot="content">
+
+        <p tal:condition="search_failed">
+          Sorry, no results found for your search.
+        </p>
+
+        <p tal:condition="not:changes">
+          No revisions!
+        </p>
+
+        <p tal:condition="changes"
+           class="changelog">
+          <tal:b content="structure python:collapse_all_button('cl')" />
+        </p>
+
+        <table class="log-entries"
+               tal:define="subcolspan python:4-bool(all_same_author)">
+          <tr class="log-header">
+            <th class="revision-number">
+              Rev
+            </th>
+            <th>
+            </th>
+            <th class="summary">
+              Summary
+            </th>
+            <th tal:condition="not:all_same_author"
+                class="author">
+              Committer
+            </th>
+            <th class="date" colspan="2">
+              Date
+            </th>
+          </tr>
+          <tal:block tal:repeat="entry changes">
+            <a tal:attributes="name string:entry-${entry/revno}" />
+            <tr tal:attributes="class string:revision-header parity${entry/parity}">
+              <td class="revision-number">
+                <a tal:attributes="title python:'Show revision '+history.get_revno(entry.revid);
+                                   href  python:url(['/revision', entry.revid])"
+                   tal:content="python:util.trunc(entry.revno)">
+                  text
+                </a>
+              </td>
+              <td class="expand-button" tal:define="name entry/revno"
+                  tal:content="structure python:collapse_button('cl', entry.revno)">
+              </td>
+              <td class="summary">
+                <a tal:attributes="title python:'Show revision '+history.get_revno(entry.revid);
+                                   href  python:url(['/revision', entry.revid])"
+                   tal:content="entry/short_comment">
+                  text
+                </a>
+              </td>
+              <td tal:condition="not:all_same_author"
+                  tal:content="python:util.trunc(util.hide_email(entry.author), 20)"
+                  class="author"></td>
+              <td class="date">
+                  <span tal:attributes="title python:util.date_time(entry.date)"
+                        tal:content="python:util._approximatedate(entry.date)"></span>
+              </td>
+              <td class="inventory-link">
+                <a tal:attributes="href python:branch.url(['/files', entry.revid]);
+                                   title string:Files at revision ${entry/revno}">
+                  files
+                </a>
+              </td>
+            </tr>
+
+            <tr tal:attributes="class string:revision-details-block parity${entry/parity}">
+              <td colspan="2">
+              </td>
+              <td tal:attributes="colspan subcolspan">
+                <table tal:attributes="class string:revision-details hidden-details collapse-cl-${entry/revno}-content">
+                  <tr tal:repeat="merge_point entry/merge_points">
+                    <th class="children">
+                      <tal:block condition="repeat/merge_point/start">
+                        merged in:
+                      </tal:block>
+                    </th>
+                    <td class="children">
+                      <a title="Show history"
+                         tal:attributes="href python:url(['/changes'], start_revid=merge_point.revid)"
+                         tal:content="python:revno_with_nick(merge_point)">
+                        class="revlink">
+                        ${text}
+                      </a>
+                    </td>
+                  </tr>
+                  <tr tal:repeat="parent python:entry.parents[1:]">
+                    <th class="parents">
+                      <tal:block condition="repeat/parent/start">
+                        merged from:
+                      </tal:block>
+                    </th>
+                    <td class="parents">
+                      <a title="Show history"
+                         tal:attributes="href python:url(['/changes'], start_revid=parent.revid)"
+                         tal:content="python:revno_with_nick(parent)">
+                        class="revlink">
+                        ${text}
+                      </a>
+                    </td>
+                  </tr>
+                  <tr tal:condition="all_same_author">
+                    <th class="author">
+                      committed by:
+                    </th>
+                    <td class="author" tal:content="python:util.hide_email(entry.author)">
+                      ${util.hide_email(entry.author)}
+                    </td>
+                  </tr>
+                  <tr>
+                    <th class="description">
+                      description:
+                    </th>
+                    <td class="description">
+                      <tal:block repeat="line entry/comment_clean"
+                                 content="structure string:${line}&lt;br /&gt;" />
+                    </td>
+                  </tr>
+                  <tal:block content="structure python:file_change_summary(url, entry, modified_file_link_log)" />
+                </table>
+              </td>
+            </tr>
+          </tal:block>
+        </table>
+
+        <div tal:condition="python:navigation.prev_page_revid or navigation.next_page_revid"
+             class="bar">
+          <table>
+            <tr>
+              <td class="buttons">
+                <a tal:condition="navigation/prev_page_revid"
+                   tal:attributes="href navigation/prev_page_url">
+                  &lt;&lt; page
+                </a>
+	      </td>
+ 	      <td class="rbuttons" align="right">
+                <a tal:condition="navigation/next_page_revid"
+                   tal:attributes="href navigation/next_page_url">
+                  page &gt;&gt;
+                </a>
+ 	      </td>
+ 	    </tr>
+          </table>
+        </div>
+      </div>
+    </body>
+  </html>
+</tal:block>

=== added file 'loggerhead/templates/collapse-all-button.pt'
--- loggerhead/templates/collapse-all-button.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/collapse-all-button.pt	2007-08-30 13:41:20 +0000
@@ -0,0 +1,16 @@
+<tal:block>
+  <a tal:attributes="class string:hide-all collapse-${group}-hideall;
+                     href string:javascript:collapseAllDisplay('${group}','none')">
+    <img tal:attributes="src python:tg.url('/static/images/nav-small-down.gif')"
+         width="10" height="10" alt="collapse"
+         class="collapse-triangle" />
+    collapse all
+  </a>
+  <a tal:attributes="class string:hide-all collapse-${group}-showall;
+                     href string:javascript:collapseAllDisplay('${group}','${normal}')">
+    <img tal:attributes="src python:tg.url('/static/images/nav-small-right.gif')"
+         width="10" height="10" alt="expand"
+         class="collapse-triangle" />
+    expand all
+  </a>
+</tal:block>

=== added file 'loggerhead/templates/collapse-button.pt'
--- loggerhead/templates/collapse-button.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/collapse-button.pt	2007-08-29 13:31:58 +0000
@@ -0,0 +1,16 @@
+<tal:block>
+  <a tal:attributes="href string:javascript:collapseDisplay('${group}','${name}','none');
+                     class string:hide-button collapse-${group}-${name}-hide"
+     title="collapse">
+    <img tal:attributes="src python:tg.url('/static/images/nav-small-down.gif')"
+         width="10" height="10" alt="collapse"
+         class="collapse-triangle" />
+  </a>
+  <a tal:attributes="href string:javascript:collapseDisplay('${group}','${name}','${normal}');
+                     class string:show-button collapse-${group}-${name}-show"
+     title="expand">
+    <img tal:attributes="src python:tg.url('/static/images/nav-small-right.gif')"
+         witdh="10" height="10" alt="expand"
+         class="collapse-triangle" />
+  </a>
+</tal:block>

=== removed file 'loggerhead/templates/inventory.kid'
--- loggerhead/templates/inventory.kid	2008-03-10 04:22:42 +0000
+++ loggerhead/templates/inventory.kid	1970-01-01 00:00:00 +0000
@@ -1,102 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
-    py:extends="'master.kid'">
-<head>
-    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" py:replace="''"/>
-    <title> ${branch.friendly_name} : files for revision ${change.revno} </title>
-</head>
-
-<body>
-
-${navbar()}
-
-<h1> <span class="branch-name">${branch.friendly_name}</span> : files for revision ${change.revno}
-	<div class="links">
-	    <div> <b>&#8594;</b> <a href="${branch.context_url('/revision', clear=1, start_revid=revid)}">
-	        view revision </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.context_url('/changes', clear=1, start_revid=revid)}">
-	        view branch changes </a> </div>
-	</div>
-</h1>
-
-<!-- !FIXME: this is just copied verbatim from revision.kid -->
-<div class="revision-info">
-    <table>
-        <tr>
-            <th class="author">committed by:</th>
-            <td class="author"> ${util.hide_email(change.author)} </td>
-        </tr>
-        <tr>
-            <th class="date">date:</th>
-            <td class="date"> ${util.date_time(change.date)} </td>
-        </tr>
-
-        <tr py:if="len(change.merge_points) > 0">
-            <th class="children"> merged in: </th>
-            <td class="children">
-                <span py:for="child in change.merge_points">
-                    ${revision_link(child.revid, '(' + child.revno + util.if_present(' %s', child.branch_nick) + ')', clear=1, start_revid=child.revid)} <br />
-                </span>
-            </td>
-        </tr>
-        <tr py:if="len(change.parents) > 1">
-        	<th class="parents"> merged from: </th>
-        	<td class="parents">
-        	    <span py:for="parent in change.parents"><span py:if="parent.revid != change.parents[0].revid">
-        	        ${revision_link(parent.revid, '(' + parent.revno + util.if_present(' %s', parent.branch_nick) + ')', clear=1, start_revid=parent.revid)} <br />
-        	    </span></span>
-        	</td>
-        </tr>
-
-        <tr>
-            <th class="description">description:</th>
-            <td class="description"><span py:for="line in change.comment_clean">${XML(line)} <br /></span></td>
-        </tr>
-    </table>
-</div>
-
-<div class="inventory-path">
-    <b>folder:</b> <span class="folder"> ${path} </span>
-</div>
-
-<table class="inventory" width="100%">
-    <tr class="header">
-        <th class="permissions"> Permissions </th>
-        <th> <a href="${branch.context_url([ '/files', revid ], sort='filename')}">Filename</a> </th>
-        <th> Latest Rev </th>
-        <th> <a href="${branch.context_url([ '/files', revid ], sort='date')}">Last Changed</a> </th>
-        <th> <a href="${branch.context_url([ '/files', revid ], sort='size')}">Size</a> </th>
-        <th>  </th>
-        <th>  </th>
-    </tr>
-    
-    <tr class="parity1" py:if="updir">
-        <td class="permissions">drwxr-xr-x</td>
-        <td class="filename directory"><a href="${branch.context_url([ '/files', revid ], file_id=updir_file_id)}"> (up) </a></td>
-        <td> </td> <td> </td> <td> </td> <td> </td>
-    </tr>
-
-    <tr py:for="file in filelist" class="parity${file.parity}">
-        <td class="permissions"> ${util.fake_permissions(file.kind, file.executable)} </td>
-        <td class="filename ${file.kind}">
-            <a py:if="file.kind=='directory'" href="${branch.context_url([ '/files', revid ],
-                file_id=file.file_id)}">${file.filename}/</a>
-            <span py:if="file.kind=='symlink'">${file.filename}@</span>
-            <a py:if="file.kind=='file'" href="${branch.context_url([ '/annotate', revid ],
-                file_id=file.file_id)}" title="Annotate ${file.filename}">${file.filename}</a>
-        </td>
-        <td class="revision"> ${revision_link(file.revid, util.trunc(file.change.revno, 15), **util.get_context(start_revid=file.revid, filter_file_id=file.file_id))} </td>
-        <td class="date"> ${util.date_time(file.change.date)} </td>
-        <td class="size"> <span py:if="file.kind=='file'"> ${util.human_size(file.size)} </span></td>
-        <td class="changes-link"> 
-            <a href="${branch.context_url('/changes', start_revid=start_revid, file_id=file.file_id)}"
-               title="Changes affecting ${file.filename}"> changes </a>
-        </td>
-        <td class="download-link">
-            <a href="${branch.url([ '/download', file.revid, file.file_id, file.filename ])}"
-               title="Download ${file.filename} at revision ${file.change.revno}">  download </a></td>
-    </tr>
-</table>
-
-</body>
-</html>

=== added file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/inventory.pt	2008-06-11 04:53:47 +0000
@@ -0,0 +1,134 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+  <head>
+    <title metal:fill-slot="title"
+           tal:content="string:${branch/friendly_name} : files for revision ${change/revno}">
+      ${branch.friendly_name} : files for revision ${change.revno}
+    </title>
+  </head>
+  <body>
+
+    <h1 metal:fill-slot="heading">
+      <span class="branch-name" tal:content="branch/friendly_name">
+        ${branch.friendly_name}
+      </span>
+      : files for revision
+      <tal:revno content="change/revno">
+        ${change.revno}
+      </tal:revno>
+    </h1>
+
+    <div metal:fill-slot="content">
+      <div class="links">
+        <div> <b>&#8594;</b> <a tal:attributes="href python:url(['/revision'], clear=1, start_revid=revid)">
+	    view revision </a>
+        </div>
+        <div> <b>&#8594;</b> <a tal:attributes="href python:url(['/changes'], clear=1, start_revid=revid)">
+	    view branch changes </a>
+        </div>
+      </div>
+
+      <tal:revision-info replace="structure python:revisioninfo(url, branch, change)" />
+
+      <div class="inventory-path">
+        <b>folder:</b> <span class="folder" tal:content="path"> ${path} </span>
+      </div>
+
+      <table class="inventory" width="100%">
+        <tr class="header">
+          <th class="permissions">
+            Permissions
+          </th>
+          <th>
+            <a tal:attributes="href python:url(['/files', revid], sort='filename')">Filename</a>
+          </th>
+          <th>
+            Latest Rev
+          </th>
+          <th>
+            <a tal:attributes="href python:url(['/files', revid], sort='date')">Last Changed</a>
+          </th>
+          <th>
+            <a tal:attributes="href python:url(['/files', revid], sort='size')">Size</a>
+          </th>
+          <th>
+          </th>
+          <th>
+          </th>
+        </tr>
+
+        <tr class="parity1" tal:condition="python:updir">
+          <td class="permissions">drwxr-xr-x</td>
+          <td class="filename directory">
+            <a tal:attributes="href python:url(['/files', revid], file_id=updir_file_id)">
+              (up)
+            </a>
+          </td>
+          <td>
+          </td>
+          <td>
+          </td>
+          <td>
+          </td>
+          <td>
+          </td>
+        </tr>
+
+        <tr tal:repeat="file filelist"
+            tal:attributes="class string:parity${file/parity}">
+          <td class="permissions" tal:content="python:util.fake_permissions(file.kind, file.executable)">
+            ${}
+          </td>
+          <td tal:attributes="class string:filename ${file/kind}">
+            <a tal:condition="python:file.kind=='directory'"
+               tal:attributes="href python:url(['/files', revid], file_id=file.file_id)"
+               tal:content="string:${file/filename}/">
+              ${file.filename}/
+            </a>
+            <span tal:condition="python:file.kind=='symlink'"
+                  tal:content="string:${file/filename}@">
+              ${file.filename}@
+            </span>
+            <a tal:condition="python:file.kind=='file'"
+               tal:attributes="href python:url(['/annotate', revid], file_id=file.file_id);
+                               title string:Annotate ${file/filename}"
+               tal:content="string:${file/filename}">
+              ${file.filename}
+            </a>
+          </td>
+          <td class="revision">
+            <a tal:attributes="href python:url(['/revision', file.revid], start_revid=file.revid, file_id=file.file_id);
+                               title string:Show revision ${file/change/revno}"
+               tal:content="python:util.trunc(file.change.revno, 15)">
+              ${text}
+            </a>
+          </td>
+          <td class="date"
+              tal:content="python:util.date_time(file.change.date)">
+            ${}
+          </td>
+          <td class="size">
+            <span tal:condition="python:file.kind=='file'"
+                  tal:content="python:util.human_size(file.size)">
+              ${util.human_size(file.size)}
+            </span>
+          </td>
+          <td class="changes-link">
+            <a tal:attributes="href python:url(['/changes'], start_revid=start_revid, filter_file_id=file.file_id);
+                               title string:Changes affecting ${file/filename}">
+              changes
+            </a>
+          </td>
+          <td class="download-link">
+            <a tal:condition="python:file.kind=='file'"
+               tal:attributes="href python:url(['/download', file.revid, file.file_id, file.filename]);
+                               title string:Download ${file/filename} at revision ${file/change/revno}">
+              download
+            </a>
+          </td>
+        </tr>
+      </table>
+    </div>
+
+  </body>
+</html>

=== added file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/macros.pt	2008-06-10 23:21:43 +0000
@@ -0,0 +1,86 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
+          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml" metal:define-macro="main">
+  <head>
+    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" />
+
+    <title metal:define-slot="title">
+      Title.
+    </title>
+
+    <link rel="stylesheet"
+          tal:attributes="href python:tg.url('/static/css/zptstyle.css')" />
+
+    <metal:block metal:define-slot="header_extras" />
+  </head>
+
+  <body tal:attributes="onload onload|nothing">
+
+    <div class="bar" tal:condition="navigation|nothing">
+      <form tal:attributes="action python:branch.url('/changes', start_revid=getattr(navigation, 'start_revid', None), file_id=getattr(navigation, 'file_id', None))">
+        <table>
+          <tr>
+            <td>
+              <span class="buttons">
+                <a tal:attributes="href python:url('/changes', clear=1)"> changes </a>
+                <a tal:attributes="href python:url('/files', clear=1)"> files </a>
+                <span class="search">
+                  search: <input type="text" name="q" />
+                </span>
+              </span>
+            </td>
+            <td align="right" tal:condition="navigation/revid_list|nothing">
+              <span tal:condition="navigation/feed" class="rbuttons feed">
+                <a tal:attributes="href python:url(['/atom'])">
+                  <img tal:attributes="src python:tg.url('/static/images/feed-icon-16x16.gif')" />
+                </a>
+              </span>
+              <span class="navbuttons">
+                <a tal:condition="navigation/prev_page_revid"
+                   tal:attributes="href navigation/prev_page_url"
+                   title="Previous page">
+                  &#171;
+                </a>
+                <tal:arrow condition="not:navigation/prev_page_revid">
+                  &#171;
+                </tal:arrow>
+                revision <tal:b content="python:history.get_revno(revid)" />
+                (<tal:b condition="python:navigation.pagesize > 1">page </tal:b>
+                <tal:b content="navigation/page_position" /> / <tal:b content="navigation/page_count" />)
+                <a tal:condition="navigation/next_page_revid"
+                   tal:attributes="href navigation/next_page_url"
+                   title="Next page">
+                  &#187;
+                </a>
+                <tal:arrow condition="not:navigation/next_page_revid">
+                  &#187;
+                </tal:arrow>
+              </span>
+            </td>
+          </tr>
+        </table>
+      </form>
+    </div>
+
+    <h1 metal:define-slot="heading">
+      Heading
+    </h1>
+
+    <div metal:define-slot="content">
+      Content.
+    </div>
+
+    <div class="footer">
+      <span class="banner">
+        <a href="http://www.bazaar-vcs.org/">
+          <img tal:attributes="src python:tg.url('/static/images/bazaar-banner.png')" />
+        </a>
+      </span>
+      <span class="banner">
+        <a href="http://www.lag.net/loggerhead/">
+          <img tal:attributes="src python:tg.url('/static/images/loggerhead-banner.png')" />
+        </a>
+      </span>
+    </div>
+  </body>
+</html>

=== removed file 'loggerhead/templates/master.kid'
--- loggerhead/templates/master.kid	2008-03-07 10:16:51 +0000
+++ loggerhead/templates/master.kid	1970-01-01 00:00:00 +0000
@@ -1,106 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<?python import sitetemplate ?>
-<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#" py:extends="sitetemplate">
-
-<head py:match="item.tag=='{http://www.w3.org/1999/xhtml}head'" py:attrs="item.items()">
-    <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
-    <title py:replace="''">Your title goes here</title>
-    <meta py:replace="item[:]"/>
-    <style type="text/css" media="screen">
-@import "${tg.url('/static/css/style.css')}";
-    </style>
-
-<!-- !define common navbar -->
-<span py:def="navbar()" py:strip="True">
-    <!-- !requires: ${navigation: start_revid, revid, revid_list, pagesize, buttons, scan_url}, ${branch}, ${history} -->
-    <div class="bar" py:if="navigation is not None">
-        <!-- form must go OUTSIDE the table, or safari will add extra padding :( -->
-        <form action="${branch.url('/changes', start_revid=getattr(navigation, 'start_revid', None),
-              file_id=getattr(navigation, 'file_id', None))}"><table><tr>
-            <td><span class="buttons">
-                <!-- ! navbar buttons never change, from now on.  i decree it! -->
-                <a href="${branch.context_url('/changes', clear=1)}"> changes </a>
-                <a href="${branch.context_url('/files', clear=1)}"> files </a>
-                <span class="search"> search: <input type="text" name="q" /> </span>
-            </span></td>
-            <td align="right" py:if="hasattr(navigation, 'revid_list')">
-                <span py:if="hasattr(navigation, 'feed')" class="rbuttons feed">
-                    <a href="${branch.url('/atom')}">
-                    <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" /></a>
-                </span>
-                <span class="navbuttons">
-                	<span py:if="navigation.prev_page_revid">
-                	<a href="${navigation.prev_page_url}" title="Previous page"> &#171; </a>
-                    </span>
-                	<span py:if="not navigation.prev_page_revid"> &#171; </span>
-                    revision ${history.get_revno(revid)}
-                    (<span py:if="navigation.pagesize > 1">page </span>${navigation.page_position} / ${navigation.page_count})
-                	<span py:if="navigation.next_page_revid">
-                    <a href="${navigation.next_page_url}" title="Next page"> &#187; </a>
-                    </span>
-                	<span py:if="not navigation.next_page_revid"> &#187; </span>
-                </span>
-            </td>
-        </tr></table></form>
-    </div>
-</span>
-
-<span py:def="revision_link(revid, text, **overrides)" py:strip="True">
-    <a title="Show revision ${history.get_revno(revid)}"
-       href="${branch.url([ '/revision', revid ], **util.get_context(**overrides))}"> ${text} </a>
-</span>
-
-
-<!-- ! expand button functions: -->
-
-	<span py:strip="True" py:def="use_collapse_buttons()">
-		<!-- this is totally matty's fault.  i don't like javacsript. ;) -->
-		<script type="text/javascript" src="${tg.url('/static/javascript/collapse.js')}"></script>
-	</span>
-	
-	<span py:strip="True" py:def="collapse_all_button(group, normal='block')">
-	    <a class="hide-all collapse-${group}-hideall"
-	       href="javascript:collapseAllDisplay('${group}','none')">
-	        <img src="${tg.url('/static/images/nav-small-down.gif')}"
-	             width="10" height="10" alt="collapse"
-	             class="collapse-triangle" />collapse all</a>
-	    <a class="hide-all collapse-${group}-showall"
-	       href="javascript:collapseAllDisplay('${group}','${normal}')">
-	        <img src="${tg.url('/static/images/nav-small-right.gif')}"
-	             width="10" height="10" alt="expand"
-	             class="collapse-triangle" />expand all</a>
-	</span>
-	
-	<span py:strip="True" py:def="collapse_button(group, name, normal='block')">
-	    <a href="javascript:collapseDisplay('${group}','${name}','none')"
-	       class="hide-button collapse-${group}-${name}-hide" title="collapse">
-	        <img src="${tg.url('/static/images/nav-small-down.gif')}"
-	             width="10" height="10" alt="collapse"
-	             class="collapse-triangle" />
-	    </a>
-	    <a href="javascript:collapseDisplay('${group}','${name}','${normal}')"
-	       class="show-button collapse-${group}-${name}-show" title="expand">
-	    	<img src="${tg.url('/static/images/nav-small-right.gif')}"
-	    	     witdh="10" height="10" alt="expand"
-	    	     class="collapse-triangle" />
-	    </a>
-	</span>
-
-</head>
-
-<body py:match="item.tag=='{http://www.w3.org/1999/xhtml}body'"
-      py:attrs="item.items()">
-
-    <div py:replace="[item.text]+item[:]"/>
-
-	<!-- End of main_content -->
-	
-	<div class="footer">
-	    <span class="banner"><a href="http://www.bazaar-vcs.org/">
-	    <img src="${tg.url('/static/images/bazaar-banner.png')}" /></a></span>
-	    <span class="banner"><a href="http://www.lag.net/loggerhead/">
-	    <img src="${tg.url('/static/images/loggerhead-banner.png')}" /></a></span>
-	</div>
-</body>
-
-</html>

=== added file 'loggerhead/templates/modified-file-link-log.pt'
--- loggerhead/templates/modified-file-link-log.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/modified-file-link-log.pt	2008-06-11 02:19:53 +0000
@@ -0,0 +1,6 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<a tal:attributes="href python:'javascript:diff_url(\'' + url(['/revision', entry.revid]) + '#' + item.filename + '\')';
+                   title string:Annotate ${item/filename}"
+   tal:content="item/filename" class="filename">
+  item/filename
+</a>

=== added file 'loggerhead/templates/modified-file-link-rev.pt'
--- loggerhead/templates/modified-file-link-rev.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/modified-file-link-rev.pt	2008-06-11 00:02:50 +0000
@@ -0,0 +1,14 @@
+<tal:block>
+  <a class="collapse-style-sbs-content filename"
+     tal:attributes="href  string:#${item/filename}-s;
+                     title string:Jump to ${item/filename} below"
+     tal:content="item/filename" >
+    item/filename
+  </a>
+  <a class="collapse-style-unified-content filename"
+     tal:attributes="href  string:#${item/filename}-u;
+                     title string:Jump to ${item/filename} below"
+     tal:content="item/filename" >
+    item/filename
+  </a>
+</tal:block>

=== removed file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2008-03-09 00:22:47 +0000
+++ loggerhead/templates/revision.kid	1970-01-01 00:00:00 +0000
@@ -1,242 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
-    py:extends="'master.kid'">
-<head>
-    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" py:replace="''"/>
-    <title> ${branch.friendly_name} : revision ${change.revno} </title>
-    
-    <span py:strip="True" py:def="file_link(filename, file_id)">
-        <a href="${branch.url([ '/annotate', revid ],
-            **util.get_context(file_id=file_id))}" title="Annotate ${filename}">${filename}</a>
-    </span>
-    <span py:strip="True" py:def="file_link_rev(filename, file_id, revid)">
-        <a href="${branch.url([ '/annotate', revid ],
-            **util.get_context(file_id=file_id))}" title="Annotate ${filename} at revision ${history.get_revno(revid)}">${filename}</a>
-    </span>
-    
-    <span py:replace="use_collapse_buttons()"></span>
-    
-    <script type="text/javascript"> <!--
-    function show_sbs(path) {
-        collapseDisplay('style', 'sbs', 'table');
-        collapseDisplay('style', 'unified', 'none');
-        document.cookie='diff=sbs; path=' + path; 
-    }
-    function show_unified(path) {
-        collapseDisplay('style', 'unified', 'table');
-        collapseDisplay('style', 'sbs', 'none');
-        document.cookie='diff=unified; path=' + path; 
-    }
-    function load(path) {
-        sortCollapseElements();
-        if (document.cookie.indexOf('diff=unified') >= 0) {
-            show_unified(path);
-        }
-        if (document.cookie.indexOf('stop=1') >= 0) {
-            return;
-        }
-        if (document.cookie.indexOf('diff=unified') >= 0) {
-            window.location.hash = window.location.hash;
-            document.cookie='stop=1; max-age=5; path=' + window.location.pathname;
-        } 
-    }
-    // --> </script>
-</head>
-
-<body onload="javascript:load('${branch.url('/')}')">
-
-${navbar()}
-
-<h1> <span class="branch-name">${branch.friendly_name}</span> : revision ${change.revno}
-    <span py:if="compare_revid is not None"> (compared to revision ${history.get_revno(compare_revid)}) </span>
-    
-    <div class="links">
-        <div> <b>&#8594;</b> <a href="${branch.context_url([ '/files', revid ])}">
-            browse files at revision ${change.revno} </a> </div>
-        <div> <b>&#8594;</b> <a href="${branch.context_url('/changes', start_revid=revid)}">
-            view history from revision ${change.revno} </a> </div>
-        <span py:if="compare_revid is not None" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/bundle', revid, compare_revid, 'bundle.txt' ])}">
-                download bundle from ${history.get_revno(compare_revid)} to ${change.revno} </a> </div>
-        </span>
-        <span py:if="(remember is not None) and (compare_revid is None) and (revid != remember)" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], compare_revid=remember)}">
-                compare with revision ${history.get_revno(remember)} </a></div>
-        </span>
-        <span py:if="remember != revid" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], remember=revid, compare_revid=None)}">
-                compare with another revision </a></div>
-        </span>
-        <span py:if="compare_revid is not None" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', compare_revid ], remember=revid, compare_revid=revid)}">
-                reverse the comparison (${change.revno} to ${history.get_revno(compare_revid)})</a></div>
-            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], remember=None, compare_revid=None)}">
-                stop comparing with revision ${history.get_revno(compare_revid)} </a></div>
-        </span>
-    </div>
-</h1>
- 
-<div class="revision-info">
-    <table>
-        <tr>
-            <th class="author">committed by:</th>
-            <td class="author"> ${util.hide_email(change.author)} </td>
-        </tr>
-        <tr>
-            <th class="date">date:</th>
-            <td class="date"> ${util.date_time(change.date)} </td>
-        </tr>
-
-        <tr py:if="len(change.merge_points) > 0">
-            <th class="children"> merged in: </th>
-            <td class="children">
-                <span py:for="child in change.merge_points">
-                    ${revision_link(child.revid, '(' + child.revno + util.if_present(' %s', child.branch_nick) + ')', clear=1, start_revid=child.revid)} <br /> 
-                </span>
-            </td>
-        </tr>
-        <tr py:if="len(change.parents) > 1">
-        	<th class="parents"> merged from: </th>
-        	<td class="parents">
-        	    <span py:for="parent in change.parents"><span py:if="parent.revid != change.parents[0].revid">
-        	        ${revision_link(parent.revid, '(' + parent.revno + util.if_present(' %s', parent.branch_nick) + ')', clear=1, start_revid=parent.revid)} <br />
-        	    </span></span>
-        	</td>
-        </tr>
-
-        <tr>
-            <th class="description">description:</th>
-            <td class="description"><span py:for="line in change.comment_clean">${XML(line)} <br /></span> </td>
-        </tr>
-        
-        <tr class="divider"> <th></th> <td></td> </tr>
-        
-        <tr py:if="change.changes.added">
-            <th class="files"> files added: </th>
-            <td class="files"> <span py:for="filename, file_id in change.changes.added" class="filename">${file_link(filename, file_id)} <br /></span> </td>
-        </tr>
-        <tr py:if="change.changes.removed">
-            <th class="files"> files removed: </th>
-            <td class="files"> <span py:for="filename, file_id in change.changes.removed" class="filename">
-                ${file_link_rev(filename, file_id, change.parents[0].revid)} <br /></span>
-            </td>
-        </tr>
-        <tr py:if="change.changes.renamed">
-            <th class="files"> files renamed: </th>
-            <td class="files"> <span py:for="old_filename, new_filename, file_id in change.changes.renamed" class="filename">
-                ${file_link(old_filename, file_id)} => ${file_link(new_filename, file_id)}<br />
-            </span> </td>
-        </tr>
-        <tr py:if="change.changes.modified">
-            <th class="files"> files modified: </th>
-            <td class="files">
-                <span py:for="item in change.changes.modified" class="collapse-style-sbs-content">
-                    <a href="#${item.filename}-s" class="filename" title="Jump to ${item.filename} below">${item.filename}</a><br />
-                </span>
-                <span py:for="item in change.changes.modified" class="collapse-style-unified-content">
-                    <a href="#${item.filename}-u" class="filename" title="Jump to ${item.filename} below">${item.filename}</a><br />
-                </span>
-            </td>
-        </tr>
-    </table>
-</div>
-
-<table class="diff-option-buttons">
-<tr>
-    <td> ${collapse_all_button('file', 'table-row')} </td>
-
-    <td class="spacey">
-        <a href="javascript:show_sbs('${branch.url('/')}')" class="hide-all collapse-style-sbs-show" title="collapse">
-            <img src="${tg.url('/static/images/nav-small-out.gif')}" width="22" height="10" class="collapse-triangle" />show side by side</a>
-        <a href="javascript:show_unified('${branch.url('/')}')" class="hide-all collapse-style-unified-show" title="expand">
-            <img src="${tg.url('/static/images/nav-small-in.gif')}" width="22" height="10" class="collapse-triangle" />show unified diff</a>
-    </td>
-    
-    <td class="diff-key-block diff-insert"></td>
-    <td class="label"> added </td>
-    <td class="diff-key-block diff-delete"></td>
-    <td class="label"> removed</td>
-</tr>
-</table>
-
-<!-- ! i'm not a big fan of embedding python code here, but the alternatives all seem to be worse -->
-<?python uniqs={}; ?>
-
-<div class="diff" py:if="change.changes.modified">
-    <!-- ! side-by-side diff -->
-    <table class="diff-block collapse-style-sbs-content">
-        <span py:strip="True" py:for="item in change.changes.modified">
-            <tr><th class="filename" colspan="4">
-                ${collapse_button('file', util.uniq(uniqs, item.file_id), 'table-row')}
-                <a href="${branch.url([ '/annotate', change.revid ], **util.get_context(file_id=item.file_id))}"
-                    name="${item.filename}-s" title="Annotate ${item.filename}">${item.filename}</a>
-            </th></tr>
-
-            <span py:strip="True" py:for="chunk in item.sbs_chunks">
-                <tr class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content">
-                    <th class="lineno">old</th> <th></th> <th class="lineno">new</th> <th></th>
-                </tr>
-                <tr py:for="line in chunk.diff" class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content">
-                    <span py:if="line.old_lineno" py:strip="True">
-                        <td class="lineno">${line.old_lineno}</td>
-                        <td class="diff-${line.old_type}">${line.old_line}</td>
-                    </span>
-                    <span py:if="not line.old_lineno" py:strip="True">
-                        <td class="lineno-skip">${line.old_lineno}</td>
-                        <td class="diff-${line.old_type}-skip">${line.old_line}</td>
-                    </span>
-                    <span py:if="line.new_lineno" py:strip="True">
-                        <td py:if="line.new_lineno" class="lineno">${line.new_lineno}</td>
-                        <td class="diff-${line.new_type}">${line.new_line}</td>
-                    </span>
-                    <span py:if="not line.new_lineno" py:strip="True">
-                        <td py:if="not line.new_lineno" class="lineno-skip">${line.new_lineno}</td>
-                        <td class="diff-${line.new_type}-skip">${line.new_line}</td>
-                    </span>
-                </tr>
-                <tr class="diff-chunk-spacing collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <td colspan="4"> &nbsp; </td> </tr>
-            </span>
-            <tr class="diff-spacing"> <td colspan="4"> &nbsp; </td> </tr>
-        </span>
-    </table>
-    
-    <!-- ! unified diff -->
-    <table class="diff-block collapse-style-unified-content">
-        <span py:strip="True" py:for="item in change.changes.modified">
-            <tr><th class="filename" colspan="4">
-                ${collapse_button('file', util.uniq(uniqs, item.file_id), 'table-row')}
-                <a href="${branch.url([ '/annotate', change.revid ], **util.get_context(file_id=item.file_id))}"
-                    name="${item.filename}-u" title="Annotate ${item.filename}">${item.filename}</a>
-            </th></tr>
-
-            <span py:strip="True" py:for="chunk in item.chunks">
-                <tr class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <th class="lineno">old</th> <th class="lineno">new</th> <th></th> <th></th> </tr>
-                <tr py:for="line in chunk.diff" class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content">
-                    <td class="lineno">${line.old_lineno}</td>
-                    <td class="lineno">${line.new_lineno}</td>
-                    <td class="diff-${line.type} text">${line.line}</td>
-                    <td> </td>
-                </tr>
-                <tr class="diff-chunk-spacing collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <td colspan="4"> &nbsp; </td> </tr>
-            </span>
-            <tr class="diff-spacing"> <td colspan="4"> &nbsp; </td> </tr>
-        </span>
-    </table>
-
-</div>
-
-<div py:if="navigation.prev_page_revid or navigation.next_page_revid" class="bar">
-    <table>
-        <tr>
-        <td class="buttons">
-            <a py:if="navigation.prev_page_revid" href="${navigation.prev_page_url}"> &lt; revision ${history.get_revno(navigation.prev_page_revid)} </a>
-        </td>
-        <td class="rbuttons" align="right">
-            <a py:if="navigation.next_page_revid" href="${navigation.next_page_url}"> revision ${history.get_revno(navigation.next_page_revid)} &gt; </a>
-        </td>
-	</tr>
-    </table>
-</div>
-
-</body>
-</html>

=== added file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/revision.pt	2008-06-11 03:17:41 +0000
@@ -0,0 +1,295 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<tal:block define="onload string:load()">
+  <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+    <head>
+      <title metal:fill-slot="title"
+             tal:content="string:${branch/friendly_name} : revision ${change/revno}">
+        ${branch.friendly_name} : revision ${change.revno}
+      </title>
+
+      <metal:block fill-slot="header_extras">
+        <script type="text/javascript"
+                tal:attributes="src python:tg.url('/static/javascript/collapse.js')">
+        </script>
+      </metal:block>
+    </head>
+
+    <body>
+
+      <h1 metal:fill-slot="heading">
+        <span class="branch-name" tal:content="branch/friendly_name">
+          ${branch.friendly_name}
+        </span>
+        : revision
+        <tal:revno content="change/revno">
+          ${change.revno}
+        </tal:revno>
+        <tal:compare-to condition="python:compare_revid is not None">
+          (compared to revision <tal:b content="python:history.get_revno(compare_revid)" />)
+        </tal:compare-to>
+      </h1>
+
+      <div metal:fill-slot="content">
+        <div class="links">
+          <div>
+            <b>&#8594;</b>
+            <a tal:attributes="href python:url(['/files', revid])">
+              browse files at revision <tal:b content="change/revno" />
+            </a>
+          </div>
+          <div>
+            <b>&#8594;</b>
+            <a tal:attributes="href python:url(['/changes'], start_revid=revid)">
+              view history from revision <tal:b content="change/revno" />
+            </a>
+          </div>
+          <div tal:condition="python:compare_revid is not None" >
+            <b>&#8594;</b>
+            <a tal:attributes="href python:branch.url(['/bundle', revid, compare_revid, 'bundle.txt'])">
+                download bundle from <tal:b content="python:history.get_revno(compare_revid)" /> to <tal:b content="change/revno" />
+            </a>
+          </div>
+          <div tal:condition="python:(remember is not None) and (compare_revid is None) and (revid != remember)" >
+            <b>&#8594;</b>
+            <a tal:attributes="href python:url(['/revision', revid], compare_revid=remember)">
+              compare with revision <tal:b content="python:history.get_revno(remember)" />
+            </a>
+          </div>
+          <div  tal:condition="python:remember != revid">
+            <b>&#8594;</b>
+            <a tal:attributes="href python:url(['/revision', revid], remember=revid, compare_revid=None)">
+              compare with another revision
+            </a>
+          </div>
+          <tal:we-are-comparing condition="python:compare_revid is not None">
+            <div>
+              <b>&#8594;</b>
+              <a tal:attributes="href python:url([ '/revision', compare_revid ], remember=revid, compare_revid=revid)">
+                reverse the comparison (<tal:b content="change/revno" /> to <tal:b content="python:history.get_revno(compare_revid)" />)
+              </a>
+            </div>
+            <div>
+              <b>&#8594;</b>
+              <a tal:attributes="href python:url(['/revision', revid], remember=None, compare_revid=None)">
+                stop comparing with revision <tal:b content="python:history.get_revno(compare_revid)" />
+              </a>
+            </div>
+          </tal:we-are-comparing>
+        </div>
+
+        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, modified_file_link_rev)" />
+
+        <table class="diff-option-buttons">
+          <tr>
+            <td>
+              <tal:b content="structure python:collapse_all_button('file', 'table-row')" />
+            </td>
+
+            <td class="spacey">
+              <a href="javascript:show_sbs()"
+                 class="hide-all collapse-style-sbs-show"
+                 title="collapse">
+                <img tal:attributes="src python:tg.url('/static/images/nav-small-out.gif')"
+                     width="22" height="10" class="collapse-triangle" />
+                show side by side
+              </a>
+              <a href="javascript:show_unified()"
+                 class="hide-all collapse-style-unified-show"
+                 title="expand">
+                <img tal:attributes="src python:tg.url('/static/images/nav-small-in.gif')"
+                     width="22" height="10" class="collapse-triangle" />
+                show unified diff
+              </a>
+            </td>
+
+            <td class="diff-key-block diff-insert">
+            </td>
+            <td class="label">
+              added
+            </td>
+            <td class="diff-key-block diff-delete">
+            </td>
+            <td class="label">
+              removed
+            </td>
+          </tr>
+        </table>
+
+        <div class="diff"
+             tal:condition="change/changes/modified"
+             tal:define="uniqs python:{}">
+          <!-- ! side-by-side diff -->
+          <table class="diff-block collapse-style-sbs-content">
+            <tal:block repeat="item change/changes/modified">
+              <tal:u define="u python:util.uniq(uniqs, item.file_id)">
+                <tr>
+                  <th class="filename" colspan="4">
+                    <tal:b content="structure python:collapse_button('file', u, 'table-row')" />
+                    <a tal:attributes="href  python:url(['/annotate', change.revid], file_id=item.file_id);
+                                       name  string:${item/filename}-s;
+                                       title string:Annotate ${item/filename}"
+                       tal:content="item/filename">
+                      ${item.filename}
+                    </a>
+                  </th>
+                </tr>
+
+                <tal:block repeat="chunk item/sbs_chunks">
+                  <tr tal:attributes="class string:diff-chunk collapse-file-${u}-content">
+                    <th class="lineno">
+                      old
+                    </th>
+                    <th>
+                    </th>
+                    <th class="lineno">
+                      new
+                    </th>
+                    <th>
+                    </th>
+                  </tr>
+                  <tr tal:repeat="line chunk/diff"
+                      tal:attributes="class string:diff-chunk collapse-file-${u}-content">
+                    <tal:block condition="line/old_lineno">
+                      <td class="lineno"
+                          tal:content="line/old_lineno">
+                        ${line.old_lineno}
+                      </td>
+                      <td tal:attributes="class string:diff-${line/old_type}"
+                          tal:content="line/old_line">
+                        ${line.old_line}
+                      </td>
+                    </tal:block>
+                    <tal:block condition="not:line/old_lineno">
+                      <td class="lineno-skip"
+                          tal:content="line/old_lineno">
+                        ${line.old_lineno}
+                      </td>
+                      <td tal:attributes="class string:diff-${line/old_type}-skip"
+                          tal:content="line/old_line">
+                        ${line.old_line}
+                      </td>
+                    </tal:block>
+                    <tal:block condition="line/new_lineno">
+                      <td class="lineno"
+                          tal:content="line/new_lineno">
+                        ${line.new_lineno}
+                      </td>
+                      <td tal:attributes="class string:diff-${line/new_type}"
+                          tal:content="line/new_line">
+                        ${line.new_line}
+                      </td>
+                    </tal:block>
+                    <tal:block condition="not:line/new_lineno">
+                      <td class="lineno-skip"
+                          tal:content="line/new_lineno">
+                        ${line.new_lineno}
+                      </td>
+                      <td tal:attributes="class string:diff-${line/new_type}-skip"
+                          tal:content="line/new_line">
+                        ${line.new_line}
+                      </td>
+                    </tal:block>
+                  </tr>
+                  <tr tal:attributes="class string:diff-chunk-spacing collapse-file-${u}-content">
+                    <td colspan="4">
+                      &nbsp;
+                    </td>
+                  </tr>
+                </tal:block>
+                <tr class="diff-spacing">
+                  <td colspan="4">
+                    &nbsp;
+                  </td>
+                </tr>
+              </tal:u>
+            </tal:block>
+          </table>
+
+          <!-- ! unified diff -->
+          <table class="diff-block collapse-style-unified-content">
+            <tal:block repeat="item change/changes/modified">
+              <tal:u define="u python:util.uniq(uniqs, item.file_id)">
+                <tr>
+                  <th class="filename" colspan="4">
+                    <tal:b content="structure python:collapse_button('file', u, 'table-row')" />
+                    <a tal:attributes="href  python:url(['/annotate', change.revid], file_id=item.file_id);
+                                       name  string:${item/filename}-u;
+                                       title string:Annotate ${item/filename}"
+                       tal:content="item/filename">
+                      ${item.filename}
+                    </a>
+                  </th>
+                </tr>
+
+                <tal:block repeat="chunk item/chunks">
+                  <tr tal:attributes="class string:diff-chunk collapse-file-${u}-content">
+                    <th class="lineno">
+                      old
+                    </th>
+                    <th class="lineno">
+                      new
+                    </th>
+                    <th>
+                    </th>
+                    <th>
+                    </th>
+                  </tr>
+                  <tr tal:repeat="line chunk/diff"
+                      tal:attributes="class string:diff-chunk collapse-file-${u}-content">
+                    <td class="lineno"
+                        tal:content="line/old_lineno">
+                      ${line.old_lineno}
+                    </td>
+                    <td class="lineno"
+                        tal:content="line/new_lineno">
+                      ${line.new_lineno}
+                    </td>
+                    <td tal:attributes="class string:diff-${line/type} text"
+                        tal:content="line/line">
+                      ${line.line}
+                    </td>
+                    <td>
+                    </td>
+                  </tr>
+                  <tr tal:attributes="class string:diff-chunk-spacing collapse-file-${u}-content">
+                    <td colspan="4">
+                      &nbsp;
+                    </td>
+                  </tr>
+                </tal:block>
+                <tr class="diff-spacing">
+                  <td colspan="4">
+                    &nbsp;
+                  </td>
+                </tr>
+              </tal:u>
+            </tal:block>
+          </table>
+        </div>
+
+        <div tal:condition="python:navigation.prev_page_revid or navigation.next_page_revid"
+             class="bar">
+          <table>
+            <tr>
+              <td class="buttons">
+                <a tal:condition="navigation/prev_page_revid"
+                   tal:attributes="href navigation/prev_page_url">
+                  &lt; revision
+                  <tal:r content="python:history.get_revno(navigation.prev_page_revid)" />
+                </a>
+	      </td>
+ 	      <td class="rbuttons" align="right">
+                <a tal:condition="navigation/next_page_revid"
+                   tal:attributes="href navigation/next_page_url">
+                  revision
+                  <tal:r content="python:history.get_revno(navigation.next_page_revid)" />
+                  &gt;
+                </a>
+ 	      </td>
+ 	    </tr>
+          </table>
+        </div>
+      </div>
+    </body>
+  </html>
+</tal:block>

=== added file 'loggerhead/templates/revisionfilechanges.pt'
--- loggerhead/templates/revisionfilechanges.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/revisionfilechanges.pt	2007-09-05 18:40:01 +0000
@@ -0,0 +1,67 @@
+<tal:block>
+  <tr tal:repeat="added entry/changes/added">
+    <th class="files">
+      <tal:block condition="repeat/added/start">
+        files added:
+      </tal:block>
+    </th>
+    <td class="files"
+        tal:define="filename python:added[0];
+                    file_id  python:added[1]">
+      <a tal:attributes="href  python:url(['/annotate', entry.revid], file_id=file_id);
+                         title string:Annotate ${filename}"
+         tal:content="filename" class="filename">
+        filename
+      </a>
+    </td>
+  </tr>
+  <tr tal:repeat="removed entry/changes/removed">
+    <th class="files">
+      <tal:block condition="repeat/removed/start">
+        files removed:
+      </tal:block>
+    </th>
+    <td class="files"
+        tal:define="filename python:removed[0];
+                    file_id  python:removed[1]">
+      <a tal:attributes="href  python:url(['/annotate', entry.revid], file_id=file_id);
+                         title string:Annotate ${filename}"
+         tal:content="filename" class="filename">
+        filename
+      </a>
+    </td>
+  </tr>
+  <tr tal:repeat="renamed entry/changes/renamed">
+    <th class="files">
+      <tal:block condition="repeat/renamed/start">
+        files renamed:
+      </tal:block>
+    </th>
+    <td class="files"
+        tal:define="old_filename python:renamed[0];
+                    new_filename python:renamed[1];
+                    file_id      python:renamed[2]">
+      <a tal:attributes="href  python:url(['/annotate', entry.revid], file_id=file_id);
+                         title string:Annotate ${old_filename}"
+         tal:content="old_filename" class="filename">
+        old_filename
+      </a>
+      =>
+      <a tal:attributes="href  python:url(['/annotate', entry.revid], file_id=file_id);
+                         title string:Annotate ${new_filename}"
+         tal:content="new_filename">
+        new_filename
+      </a>
+    </td>
+  </tr>
+  <tr tal:repeat="item entry/changes/modified" class="filename">
+    <th class="files">
+      <tal:block condition="repeat/item/start">
+        files modified:
+      </tal:block>
+    </th>
+    <td class="files">
+        <tal:b content="structure python:modified_file_link(url, entry, item)" />
+    </td>
+  </tr>
+</tal:block>

=== added file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/revisioninfo.pt	2008-05-29 05:58:01 +0000
@@ -0,0 +1,78 @@
+<div class="revision-info">
+  <table>
+    <tr>
+      <th class="author">
+        committed by:
+      </th>
+      <td class="author"
+          tal:content="python:util.hide_email(change.author)">
+        author@email
+      </td>
+    </tr>
+    <tr>
+      <th class="date">
+        date:
+      </th>
+      <td class="date"
+          tal:content="python:util.date_time(change.date)">
+        date
+      </td>
+    </tr>
+
+    <tr tal:repeat="merge_point change/merge_points">
+      <th class="children">
+        <tal:block condition="repeat/merge_point/start">
+          merged in:
+        </tal:block>
+      </th>
+      <td class="children">
+        <a title="Show history"
+           tal:attributes="title string:Show revision ${merge_point/revno};
+                           href python:url(['/revision', merge_point.revid],
+                                           clear=1, start_revid=merge_point.revid)"
+           tal:content="python:revno_with_nick(merge_point)">
+          ${text}
+        </a>
+      </td>
+    </tr>
+
+    <tr tal:repeat="parent python:change.parents[1:]">
+      <th class="parents">
+        <tal:block condition="repeat/parent/start">
+          merged from:
+        </tal:block>
+      </th>
+      <td class="parents">
+        <a title="Show history"
+           tal:attributes="title string:Show revision ${parent/revno};
+                             href python:url(['/revision', parent.revid], clear=1, start_revid=parent.revid)"
+           tal:content="python:revno_with_nick(parent)">
+          class="revlink">
+          ${text}
+        </a>
+      </td>
+    </tr>
+
+    <tr>
+      <th class="description">
+        description:
+      </th>
+      <td class="description">
+        <span tal:repeat="line change/comment_clean">
+          <tal:line replace="structure line" />
+          <br />
+        </span>
+      </td>
+    </tr>
+
+    <tal:block condition="nocall:modified_file_link">
+      <tr class="divider">
+        <th>
+        </th>
+        <td>
+        </td>
+      </tr>
+      <tal:block content="structure python:file_change_summary(url, change, modified_file_link)" />
+    </tal:block>
+  </table>
+</div>

=== modified file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	2007-09-10 13:43:45 +0000
+++ loggerhead/tests/test_corners.py	2008-04-18 03:22:47 +0000
@@ -1,3 +1,4 @@
+import re
 import os
 
 import cherrypy
@@ -93,3 +94,25 @@
         # It's not much of an assertion, but we only really care about
         # "assert not crashed".
         assert 'myfilename' in cherrypy.response.body[0]
+
+    def test_revision_size_limit(self):
+        self.createBranch()
+        msg = 'a very exciting commit message'
+        self.addFileAndCommit('myfilename', msg)
+
+        file_in_branch = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
+        file_in_branch.write('\n'.join(['line']*100))
+        file_in_branch.close()
+        newrevid = self.tree.commit(message='touch 100 lines')
+
+        self.setUpLoggerhead()
+        cherrypy.root._config['line_count_limit'] = 50
+        testutil.create_request('/project/branch/revision/'+newrevid)
+
+        match = re.search(
+            'Diff of [0-9]+ lines is too long to display richly -- limit '
+            'is 50 lines\\.',
+            cherrypy.response.body[0]
+            )
+
+        assert match is not None

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2008-03-04 22:01:33 +0000
+++ loggerhead/tests/test_simple.py	2008-05-28 22:47:44 +0000
@@ -32,7 +32,7 @@
         self.tearDown()
 
     def setUp(self):
-        #logging.basicConfig(level=logging.DEBUG)
+        logging.basicConfig(level=logging.DEBUG)
         self.bzrbranch = None
         self.old_bzrhome = None
 

=== removed file 'loggerhead/textindex.py'
--- loggerhead/textindex.py	2008-02-27 03:36:09 +0000
+++ loggerhead/textindex.py	1970-01-01 00:00:00 +0000
@@ -1,240 +0,0 @@
-#
-# Copyright (C) 2006  Robey Pointer <robey@lag.net>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
-
-"""
-indexing of the comment text of revisions, for fast searching.
-
-two separate database files are created:
-
-    - recorded: revid -> 1 (if the revid is indexed)
-    - index: 3-letter substring -> list(revids)
-"""
-
-import os
-import re
-import time
-
-from loggerhead import util
-from loggerhead.lockfile import LockFile
-from loggerhead.changecache import FakeShelf
-
-# if any substring index reaches this many revids, replace the entry with
-# an ALL marker -- it's not worth an explicit index.
-ALL_THRESHOLD = 1000
-ALL = 'ALL'
-
-
-with_lock = util.with_lock('_lock')
-
-
-def normalize_string(s):
-    """
-    remove any punctuation and normalize all whitespace to a single space.
-    """
-    s = util.to_utf8(s).lower()
-    # remove apostrophes completely.
-    s = re.sub(r"'", '', s)
-    # convert other garbage into space
-    s = re.sub(r'[^\w\d]', ' ', s)
-    # compress multiple spaces into one.
-    s = re.sub(r'\s{2,}', ' ', s)
-    # and finally remove leading/trailing whitespace
-    s = s.strip()
-    return s
-
-
-class TextIndex (object):
-    def __init__(self, history, cache_path):
-        self.history = history
-        self.log = history.log
-
-        if not os.path.exists(cache_path):
-            os.mkdir(cache_path)
-
-        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded.sql')
-        self._index_filename = os.path.join(cache_path, 'textindex.sql')
-
-        # use a lockfile since the cache folder could be shared across different processes.
-        self._lock = LockFile(os.path.join(cache_path, 'index-lock'))
-        self._closed = False
-
-        self.log.info('Using search index; %d entries.', len(self))
-
-    def _index(self):
-        return FakeShelf(self._index_filename)
-
-    def _recorded(self):
-        return FakeShelf(self._recorded_filename)
-
-    def _is_indexed(self, revid, recorded):
-        return recorded.get(util.to_utf8(revid)) is not None
-
-    @with_lock
-    def is_indexed(self, revid):
-        recorded = self._recorded()
-        try:
-            return self._is_indexed(revid, recorded)
-        finally:
-            recorded.close()
-
-    @with_lock
-    def __len__(self):
-        recorded = self._recorded()
-        try:
-            return recorded.count()
-        finally:
-            recorded.close()
-
-    @with_lock
-    def close(self):
-        self._closed = True
-
-    @with_lock
-    def closed(self):
-        return self._closed
-
-    @with_lock
-    def flush(self):
-        pass
-
-    @with_lock
-    def full(self):
-        recorded = self._recorded()
-        last_revid = util.to_utf8(self.history.last_revid)
-        try:
-            return (recorded.count() >= len(self.history.get_revision_history())
-                    and recorded.get(last_revid) is not None)
-        finally:
-            recorded.close()
-
-    def _index_change(self, change, recorded, index):
-        """
-        currently, only indexes the 'comment' field.
-        """
-        comment = normalize_string(change.comment)
-        if len(comment) < 3:
-            return
-        for i in xrange(len(comment) - 2):
-            sub = comment[i:i + 3]
-            orig = revid_set = index.get(sub)
-            if revid_set is None:
-                revid_set = set()
-            elif revid_set == ALL:
-                # this entry got too big
-                continue
-            revid_set.add(change.revid)
-            if len(revid_set) > ALL_THRESHOLD:
-                revid_set = ALL
-            if orig is not None:
-                index.update([(sub, revid_set)], commit=False)
-            else:
-                index.add([(sub, revid_set)], commit=False)
-
-        recorded.add([(util.to_utf8(change.revid), True)], commit=False)
-
-    @with_lock
-    def index_changes(self, revid_list):
-        recorded = self._recorded()
-        index = self._index()
-        try:
-            revid_list = [r for r in revid_list if not self._is_indexed(r, recorded)]
-            change_list = self.history.get_changes(revid_list)
-            for change in change_list:
-                self._index_change(change, recorded, index)
-        finally:
-            index.close(commit=True)
-            recorded.close(commit=True)
-
-    @with_lock
-    def find(self, text, revid_list=None):
-        index = self._index()
-        try:
-            text = normalize_string(text)
-            if len(text) < 3:
-                return []
-
-            total_set = None
-            if revid_list is not None:
-                total_set = set(revid_list)
-            seen_all = False
-
-            for i in xrange(len(text) - 2):
-                sub = text[i:i + 3]
-                revid_set = index.get(sub)
-                if revid_set is None:
-                    # zero matches, stop here.
-                    return []
-                if revid_set == ALL:
-                    # skip
-                    seen_all = True
-                    continue
-                if total_set is None:
-                    total_set = revid_set
-                else:
-                    total_set.intersection_update(revid_set)
-                if len(total_set) == 0:
-                    return []
-        finally:
-            index.close()
-
-        # tricky: if seen_all is True, one of the substring indices was ALL
-        # (in other words, unindexed), so our results are actually a superset
-        # of the exact answer.
-        #
-        # if we cared, we could do a direct match on the result set and cull
-        # out any that aren't actually matches.  for now, i'm gonna say that
-        # we DON'T care, and if one of the substrings hit ALL, there's a small
-        # chance that we'll give a few false positives.
-        return total_set
-
-    def check_rebuild(self, max_time=3600):
-        """
-        check if there are any un-indexed revisions, and if so, index them.
-        but don't spend longer than C{max_time} on it.
-        """
-        if self.closed() or self.full():
-            # all done
-            return
-
-        self.log.info('Building search index...')
-        work = list(self.history.get_revision_history())
-        start_time = time.time()
-        last_update = time.time()
-        count = 0
-
-        jump = 100
-        for i in xrange(0, len(work), jump):
-            r = work[i:i + jump]
-            self.index_changes(r)
-            if self.closed():
-                return
-
-            count += jump
-            now = time.time()
-            if now - start_time > 3600:
-                # there's no point working for hours.  eventually we might even
-                # hit the next re-index interval, which would suck mightily.
-                self.log.info('Search indexing has worked for an hour; giving up for now.')
-                return
-            if now - last_update > 60:
-                self.log.info('Search indexing continues: %d/%d' % (min(count, len(work)), len(work)))
-                last_update = time.time()
-            # give someone else a chance at the lock
-            time.sleep(1)
-        self.log.info('Search index completed.')
-        self.flush()

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-03-10 04:22:42 +0000
+++ loggerhead/util.py	2008-06-16 09:41:35 +0000
@@ -17,7 +17,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-from elementtree import ElementTree as ET
+try:
+    from xml.etree import ElementTree as ET
+except ImportError:
+    from elementtree import ElementTree as ET
 
 import base64
 import cgi
@@ -34,6 +37,13 @@
 
 log = logging.getLogger("loggerhead.controllers")
 
+def fix_year(year):
+    if year < 70:
+        year += 2000
+    if year < 100:
+        year += 1900
+    return year
+
 # Display of times.
 
 # date_day -- just the day
@@ -103,6 +113,7 @@
 
 
 def approximatedate(date):
+    #FIXME: Returns an object instead of a string
     return _wrap_with_date_time_title(date, _approximatedate(date))
 
 

=== modified file 'start-loggerhead.py'
--- start-loggerhead.py	2008-03-08 22:55:27 +0000
+++ start-loggerhead.py	2008-06-16 08:49:28 +0000
@@ -110,10 +110,6 @@
     
     Root = Root(config)
     
-    # re-index every 6 hours
-    index_freq = config.get('cache_rebuild_frequency', 6 * 3600)
-    turbogears.scheduler.add_interval_task(initialdelay=1, interval=index_freq, action=Root._check_rebuild)
-
     try:
         turbogears.start_server(Root)
     finally:

=== added directory 'turbosimpletal'
=== added file 'turbosimpletal/__init__.py'
--- turbosimpletal/__init__.py	1970-01-01 00:00:00 +0000
+++ turbosimpletal/__init__.py	2008-06-10 23:38:42 +0000
@@ -0,0 +1,15 @@
+from turbosimpletal import zptsupport
+
+TurboZpt = zptsupport.TurboZpt
+
+__all__ = ["TurboZpt"]
+
+from turbogears.view import engines, stdvars
+
+engines['zpt'] = TurboZpt(stdvars)
+
+import logging
+simpleTALLogger = logging.getLogger("simpleTAL")
+simpleTALESLogger = logging.getLogger("simpleTALES")
+simpleTALLogger.setLevel(logging.INFO)
+simpleTALESLogger.setLevel(logging.INFO)

=== added directory 'turbosimpletal/tests'
=== added file 'turbosimpletal/tests/__init__.py'
=== added file 'turbosimpletal/tests/simple.pt'
--- turbosimpletal/tests/simple.pt	1970-01-01 00:00:00 +0000
+++ turbosimpletal/tests/simple.pt	2007-06-29 10:47:51 +0000
@@ -0,0 +1,8 @@
+<html>
+<head>
+<title tal:content="title">simple test page title</title>
+</head>
+<body>
+<div>Hello, <span tal:replace="name">name</span></div>
+</body>
+</html>

=== added file 'turbosimpletal/tests/test_zpt.py'
--- turbosimpletal/tests/test_zpt.py	1970-01-01 00:00:00 +0000
+++ turbosimpletal/tests/test_zpt.py	2008-06-11 03:41:46 +0000
@@ -0,0 +1,26 @@
+from turbosimpletal import TurboZpt
+from turbogears import controllers, expose, testutil
+import cherrypy
+
+RENDERED = u"<html>\n<head>\n<title>%s</title>\n</head>\n<body>\n<div>Hello, %s</div>\n</body>\n</html>"
+
+def test_template_lookup():
+    tc = TurboZpt()
+    template = tc.load_template("turbosimpletal.tests.simple")
+    assert template
+    TITLE="test"
+    NAME="World"
+    info = dict(title=TITLE, name=NAME)
+    s = template.expand(**info)
+    assert s.startswith(RENDERED % (TITLE, NAME))
+
+class TestRoot(controllers.Root):
+    @expose(html="zpt:turbosimpletal.tests.simple")
+    def index(self, name, title="test"):
+        return dict(name=name, title=title)
+
+def test_real_life_situation():
+    cherrypy.root = TestRoot()
+    NAME="Dave"
+    testutil.createRequest("/?name=%s" % NAME)
+    assert cherrypy.response.body[0].startswith(RENDERED % ("test", NAME))

=== added file 'turbosimpletal/zptsupport.py'
--- turbosimpletal/zptsupport.py	1970-01-01 00:00:00 +0000
+++ turbosimpletal/zptsupport.py	2008-06-11 03:41:46 +0000
@@ -0,0 +1,83 @@
+"TurboGears support for Zope Page Templates"
+
+import StringIO
+import logging
+import os
+import pkg_resources
+
+from simpletal import simpleTAL, simpleTALES
+
+log = logging.getLogger("turbogears.zptsupport")
+
+
+_zpt_cache = {}
+def zpt(tfile):
+    tinstance = _zpt_cache.get(tfile)
+    stat = os.stat(tfile)
+    if tinstance is None or tinstance.stat != stat:
+        tinstance = _zpt_cache[tfile] = TemplateWrapper(
+            simpleTAL.compileXMLTemplate(open(tfile)), tfile, stat)
+    return tinstance
+
+
+class TemplateWrapper(object):
+
+    def __init__(self, template, filename, stat):
+        self.template = template
+        self.filename = filename
+        self.stat = stat
+
+    def expand(self, **info):
+        context = simpleTALES.Context(allowPythonPath=1)
+        for k, v in info.iteritems():
+            context.addGlobal(k, v)
+        s = StringIO.StringIO()
+        self.template.expandInline(context, s)
+        return s.getvalue()
+
+    @property
+    def macros(self):
+        return self.template.macros
+
+
+class TurboZpt(object):
+    extension = "pt"
+
+    def __init__(self, extra_vars_func=None):
+        self.get_extra_vars = extra_vars_func
+
+    def load_template(self, classname, loadingSite=False):
+        """Searches for a template along the Python path.
+
+        Template files must end in ".pt" and be in legitimate packages.
+        Templates are automatically checked for changes and reloaded as
+        neccessary.
+        """
+        divider = classname.rfind(".")
+        if divider > -1:
+            package = classname[0:divider]
+            basename = classname[divider+1:]
+        else:
+            raise ValueError, "All templates must be in a package"
+
+        tfile = pkg_resources.resource_filename(
+            package, "%s.%s" % (basename, self.extension))
+        return zpt(tfile)
+
+    def render(self, info, format="html", fragment=False, template=None):
+        """Renders data in the desired format.
+
+        @param info: the data / context itself
+        @type info: dict
+        @para format: "html"
+        @type format: "string"
+        @para template: name of the template to use
+        @type template: string
+        """
+        tinstance = self.load_template(template)
+        log.debug("Applying template %s" % (tinstance.filename))
+        data = dict()
+        if self.get_extra_vars:
+            data.update(self.get_extra_vars())
+        data.update(info)
+        return tinstance.expand(**data).encode('utf-8')

